from __future__ import annotations

from typing import TYPE_CHECKING, AsyncGenerator, Literal, override

from mipac.abstract.action import AbstractAction
from mipac.file import MiFile
from mipac.http import HTTPClient, Route
from mipac.models.channel import Channel
from mipac.models.drive import File
from mipac.models.note import Note
from mipac.models.poll import MiPoll
from mipac.types.channel import IChannel
from mipac.types.note import INote, INoteVisibility
from mipac.types.reaction import IReactionAcceptance
from mipac.utils.format import remove_dict_missing
from mipac.utils.pagination import Pagination
from mipac.utils.util import MISSING

if TYPE_CHECKING:
    from mipac.client import ClientManager


class SharedChannelActions(AbstractAction):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        self._session: HTTPClient = session
        self._client: ClientManager = client

    async def send(
        self,
        text: str | None = None,
        visibility: INoteVisibility = "public",
        visible_user_ids: list[str] | None = None,
        cw: str | None = None,
        local_only: bool = False,
        reaction_acceptance: IReactionAcceptance = None,
        extract_mentions: bool = True,  # 元は noExtractMentions
        extract_hashtags: bool = True,  # 元は noExtractHashtags
        extract_emojis: bool = True,  # 元は noExtractEmojis
        reply_id: str | None = None,
        renote_id: str | None = None,
        files: list[MiFile | File | str] | None = None,
        poll: MiPoll | None = None,
        *,
        channel_id: str,
    ) -> Note:
        """Send a note

        Endpoint: `/api/notes/create`

        Parameters
        ----------
        text : str, optional
            Text of the note, by default None
        visibility : INoteVisibility, optional
            Visibility of the note, by default "public"
        visible_user_ids : list[str], optional
            Visible user IDs, by default None
        cw : str, optional
            CW of the note, by default None
        local_only : bool, optional
            Whether the note is local only, by default False
        reaction_acceptance : IReactionAcceptance, optional
            Reaction acceptance of the note, by default None
        extract_mentions : bool, optional
            Whether to extract mentions, by default True
        extract_hashtags : bool, optional
            Whether to extract hashtags, by default True
        extract_emojis : bool, optional
            Whether to extract emojis, by default True
        reply_id : str, optional
            Reply ID, by default None
        renote_id : str, optional
            Renote ID, by default None
        files : list[MiFile | File | str], optional
            Files, by default None
        poll : MiPoll, optional
            Poll, by default None
        channel_id : str, optional
            ID of the channel, by default None

        Returns
        -------
        Note
            Created note
        """
        return await self._client.note.action.send(
            text=text,
            cw=cw,
            files=files,
            poll=poll,
            visibility=visibility,
            visible_user_ids=visible_user_ids,
            reaction_acceptance=reaction_acceptance,
            extract_mentions=extract_mentions,
            extract_hashtags=extract_hashtags,
            extract_emojis=extract_emojis,
            reply_id=reply_id,
            renote_id=renote_id,
            channel_id=channel_id,
            local_only=local_only,
        )

    async def follow(self, *, channel_id: str) -> bool:
        """Follow a channel

        Endpoint: `/api/channels/follow`

        Parameters
        ----------
        channel_id : str, optional
            ID of the channel, by default None

        Returns
        -------
        bool
            Whether the channel is followed
        """
        data = {"channelId": channel_id}

        res: bool = await self._session.request(
            Route("POST", "/api/channels/follow"), json=data, auth=True, lower=True
        )
        return res

    async def unfollow(self, *, channel_id: str) -> bool:
        """指定したIDのチャンネルのフォローを解除します

        Endpoint: `/api/channels/unfollow`

        Parameters
        ----------
        channel_id : str | None
            対象のチャンネルID, default=None

        Returns
        -------
        bool
            フォロー解除に成功したかどうか
        """
        data = {"channelId": channel_id}

        res: bool = await self._session.request(
            Route("POST", "/api/channels/unfollow"), json=data, auth=True, lower=True
        )
        return res

    async def update(
        self,
        name: str | None = MISSING,
        description: str | None = MISSING,
        banner_id: str | None = MISSING,
        is_archived: bool | None = MISSING,
        pinned_note_ids: list[str] | None = MISSING,
        color: str | None = MISSING,
        is_sensitive: bool | None = MISSING,
        allow_renote_to_external: bool | None = MISSING,
        *,
        channel_id: str,
    ) -> Channel:
        """チャンネルの情報を更新します

        Endpoint: `/api/channels/update`

        Parameters
        ----------
        name : str | None
            チャンネル名, default=MISSING
        description : str | None
            チャンネルの説明, default=MISSING
        banner_id : str | None
            バナー画像のID, default=MISSING
        is_archived : bool | None
            チャンネルがアーカイブされているかどうか, default=MISSING
        pinned_note_ids : list[str] | None
            ピン留めするノートのIDリスト, default=MISSING
        color : str | None
            チャンネルの色, default=MISSING
        is_sensitive : bool | None
            チャンネルがセンシティブかどうか, default=MISSING
        allow_renote_to_external : bool | None
            外部へのリノートを許可するかどうか, default=MISSING
        channel_id : str | None
            対象のチャンネルID, default=None

        Returns
        -------
        Channel
            更新後のチャンネル
        """
        data = remove_dict_missing(
            {
                "channelId": channel_id,
                "name": name,
                "description": description,
                "bannerId": banner_id,
                "isArchived": is_archived,
                "pinnedNoteIds": pinned_note_ids,
                "color": color,
                "isSensitive": is_sensitive,
                "allowRenoteToExternal": allow_renote_to_external,
            }
        )
        raw_channel: IChannel = await self._session.request(
            Route("POST", "/api/channels/update"), json=data, auth=True, remove_none=False
        )
        return Channel(raw_channel=raw_channel, client=self._client)

    async def timeline(
        self,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
        since_date: int | None = None,
        until_date: int | None = None,
        *,
        channel_id: str,
    ) -> list[Note]:
        """チャンネルのタイムラインを取得します

        Endpoint: `/api/channels/timeline`

        Parameters
        ----------
        limit : int
            一度に取得する件数, default=10
        since_id : str | None
            指定したIDのノートより後のノートを取得します, default=None
        until_id : str | None
            指定したIDのノートより前のノートを取得します, default=None
        since_date : int | None
            指定した日付のノートより後のノートを取得します, default=None
        until_date : int | None
            指定した日付のノートより前のノートを取得します, default=None
        channel_id : str | None
            対象のチャンネルID, default=None

        Returns
        -------
        list[Note]
            取得したノートのリスト
        """
        if channel_id is None:
            raise ValueError("channel_id is required")

        data = {
            "channelId": channel_id,
            "limit": limit,
            "sinceId": since_id,
            "untilId": until_id,
            "sinceDate": since_date,
            "untilDate": until_date,
        }

        raw_notes: list[INote] = await self._session.request(
            Route("POST", "/api/channels/timeline"), json=data, auth=True, lower=True
        )
        return [Note(raw_note=raw_note, client=self._client) for raw_note in raw_notes]

    async def get_all_timeline(
        self,
        since_id: str | None = None,
        until_id: str | None = None,
        since_date: int | None = None,
        until_date: int | None = None,
        *,
        channel_id: str,
    ) -> AsyncGenerator[Note, None]:
        """チャンネルのタイムラインを全て取得します

        Endpoint: `/api/channels/timeline`

        Parameters
        ----------
        limit : int
            一度に取得する件数, default=10
        since_id : str | None
            指定したIDのノートより後のノートを取得します, default=None
        until_id : str | None
            指定したIDのノートより前のノートを取得します, default=None
        since_date : int | None
            指定した日付のノートより後のノートを取得します, default=None
        until_date : int | None
            指定した日付のノートより前のノートを取得します, default=None
        channel_id : str | None
            対象のチャンネルID, default=None

        Returns
        -------
        AsyncGenerator[Note, None]
            取得したノートのリスト
        """
        data = {
            "channelId": channel_id,
            "sinceId": since_id,
            "untilId": until_id,
            "sinceDate": since_date,
            "untilDate": until_date,
        }
        pagination = Pagination[INote](
            self._session, Route("POST", "/api/channels/timeline"), auth=True, json=data
        )

        while pagination.is_final is False:
            raw_notes: list[INote] = await pagination.next()
            for raw_note in raw_notes:
                yield Note(raw_note=raw_note, client=self._client)

    async def favorite(self, *, channel_id: str) -> bool:
        """指定したIDのチャンネルをお気に入りにします

        Endpoint: `/api/channels/favorite`

        Parameters
        ----------
        channel_id : str | None
            対象のチャンネルID, default=None

        Returns
        -------
        bool
            お気に入りに追加できたかどうか
        """
        data = {"channelId": channel_id}

        res: bool = await self._session.request(
            Route("POST", "/api/channels/favorite"), json=data, auth=True, lower=True
        )
        return res

    async def unfavorite(self, *, channel_id: str) -> bool:
        """指定したIDのチャンネルをお気に入りから外します

        Endpoint: `/api/channels/unfavorite`

        Parameters
        ----------
        channel_id : str | None
            対象のチャンネルID, default=None

        Returns
        -------
        bool
            お気に入りから外せたかどうか
        """
        data = {"channelId": channel_id}

        res: bool = await self._session.request(
            Route("POST", "/api/channels/unfavorite"), json=data, auth=True, lower=True
        )
        return res


class ClientChannelActions(SharedChannelActions):
    def __init__(self, channel_id: str, *, session: HTTPClient, client: ClientManager):
        self._channel_id: str = channel_id
        self._session: HTTPClient = session
        self._client: ClientManager = client

    @override
    async def send(
        self,
        text: str | None = None,
        visibility: INoteVisibility = "public",
        visible_user_ids: list[str] | None = None,
        cw: str | None = None,
        local_only: bool = False,
        reaction_acceptance: IReactionAcceptance = None,
        extract_mentions: bool = True,  # 元は noExtractMentions
        extract_hashtags: bool = True,  # 元は noExtractHashtags
        extract_emojis: bool = True,  # 元は noExtractEmojis
        reply_id: str | None = None,
        renote_id: str | None = None,
        files: list[MiFile | File | str] | None = None,
        poll: MiPoll | None = None,
        *,
        channel_id: str | None = None,
    ) -> Note:
        """Send a note

        Endpoint: `/api/notes/create`

        Parameters
        ----------
        text : str, optional
            Text of the note, by default None
        visibility : INoteVisibility, optional
            Visibility of the note, by default "public"
        visible_user_ids : list[str], optional
            Visible user IDs, by default None
        cw : str, optional
            CW of the note, by default None
        local_only : bool, optional
            Whether the note is local only, by default False
        reaction_acceptance : IReactionAcceptance, optional
            Reaction acceptance of the note, by default None
        extract_mentions : bool, optional
            Whether to extract mentions, by default True
        extract_hashtags : bool, optional
            Whether to extract hashtags, by default True
        extract_emojis : bool, optional
            Whether to extract emojis, by default True
        reply_id : str, optional
            Reply ID, by default None
        renote_id : str, optional
            Renote ID, by default None
        files : list[MiFile | File | str], optional
            Files, by default None
        poll : MiPoll, optional
            Poll, by default None
        channel_id : str, optional
            ID of the channel, by default None

        Returns
        -------
        Note
            Created note
        """
        channel_id = channel_id or self._channel_id
        return await super().send(
            text=text,
            cw=cw,
            files=files,
            poll=poll,
            visibility=visibility,
            visible_user_ids=visible_user_ids,
            reaction_acceptance=reaction_acceptance,
            extract_mentions=extract_mentions,
            extract_hashtags=extract_hashtags,
            extract_emojis=extract_emojis,
            reply_id=reply_id,
            renote_id=renote_id,
            channel_id=channel_id,
            local_only=local_only,
        )

    @override
    async def follow(self, *, channel_id: str) -> bool:
        """Follow a channel

        Endpoint: `/api/channels/follow`

        Parameters
        ----------
        channel_id : str, optional
            ID of the channel, by default None

        Returns
        -------
        bool
            Whether the channel is followed
        """
        return await super().follow(channel_id=channel_id)

    @override
    async def unfollow(self, *, channel_id: str) -> bool:
        """指定したIDのチャンネルのフォローを解除します

        Endpoint: `/api/channels/unfollow`

        Parameters
        ----------
        channel_id : str | None
            対象のチャンネルID, default=None

        Returns
        -------
        bool
            フォロー解除に成功したかどうか
        """
        return await super().unfollow(channel_id=channel_id)

    @override
    async def update(
        self,
        name: str | None = MISSING,
        description: str | None = MISSING,
        banner_id: str | None = MISSING,
        is_archived: bool | None = MISSING,
        pinned_note_ids: list[str] | None = MISSING,
        color: str | None = MISSING,
        is_sensitive: bool | None = MISSING,
        allow_renote_to_external: bool | None = MISSING,
        *,
        channel_id: str,
    ) -> Channel:
        """チャンネルの情報を更新します

        Endpoint: `/api/channels/update`

        Parameters
        ----------
        name : str | None
            チャンネル名, default=MISSING
        description : str | None
            チャンネルの説明, default=MISSING
        banner_id : str | None
            バナー画像のID, default=MISSING
        is_archived : bool | None
            チャンネルがアーカイブされているかどうか, default=MISSING
        pinned_note_ids : list[str] | None
            ピン留めするノートのIDリスト, default=MISSING
        color : str | None
            チャンネルの色, default=MISSING
        is_sensitive : bool | None
            チャンネルがセンシティブかどうか, default=MISSING
        allow_renote_to_external : bool | None
            外部へのリノートを許可するかどうか, default=MISSING
        channel_id : str | None
            対象のチャンネルID, default=None

        Returns
        -------
        Channel
            更新後のチャンネル
        """
        channel_id = channel_id or self._channel_id

        return await super().update(
            name=name,
            description=description,
            banner_id=banner_id,
            is_archived=is_archived,
            pinned_note_ids=pinned_note_ids,
            color=color,
            is_sensitive=is_sensitive,
            allow_renote_to_external=allow_renote_to_external,
            channel_id=channel_id,
        )

    @override
    async def timeline(
        self,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
        since_date: int | None = None,
        until_date: int | None = None,
        *,
        channel_id: str,
    ) -> list[Note]:
        """チャンネルのタイムラインを取得します

        Endpoint: `/api/channels/timeline`

        Parameters
        ----------
        limit : int
            一度に取得する件数, default=10
        since_id : str | None
            指定したIDのノートより後のノートを取得します, default=None
        until_id : str | None
            指定したIDのノートより前のノートを取得します, default=None
        since_date : int | None
            指定した日付のノートより後のノートを取得します, default=None
        until_date : int | None
            指定した日付のノートより前のノートを取得します, default=None
        channel_id : str | None
            対象のチャンネルID, default=None

        Returns
        -------
        list[Note]
            取得したノートのリスト
        """
        channel_id = channel_id or self._channel_id

        return await super().timeline(
            limit=limit,
            since_id=since_id,
            until_id=until_id,
            since_date=since_date,
            until_date=until_date,
            channel_id=channel_id,
        )

    @override
    async def get_all_timeline(
        self,
        since_id: str | None = None,
        until_id: str | None = None,
        since_date: int | None = None,
        until_date: int | None = None,
        *,
        channel_id: str | None = None,
    ) -> AsyncGenerator[Note, None]:
        """チャンネルのタイムラインを全て取得します

        Endpoint: `/api/channels/timeline`

        Parameters
        ----------
        limit : int
            一度に取得する件数, default=10
        since_id : str | None
            指定したIDのノートより後のノートを取得します, default=None
        until_id : str | None
            指定したIDのノートより前のノートを取得します, default=None
        since_date : int | None
            指定した日付のノートより後のノートを取得します, default=None
        until_date : int | None
            指定した日付のノートより前のノートを取得します, default=None
        channel_id : str | None
            対象のチャンネルID, default=None

        Returns
        -------
        AsyncGenerator[Note, None]
            取得したノートのリスト
        """
        channel_id = channel_id or self._channel_id

        async for i in super().get_all_timeline(
            since_id=since_id,
            until_id=until_id,
            since_date=since_date,
            until_date=until_date,
            channel_id=channel_id,
        ):
            yield i

    @override
    async def favorite(self, *, channel_id: str) -> bool:
        """指定したIDのチャンネルをお気に入りにします

        Endpoint: `/api/channels/favorite`

        Parameters
        ----------
        channel_id : str | None
            対象のチャンネルID, default=None

        Returns
        -------
        bool
            お気に入りに追加できたかどうか
        """
        channel_id = channel_id or self._channel_id

        return await super().favorite(channel_id=channel_id)

    @override
    async def unfavorite(self, *, channel_id: str) -> bool:
        """指定したIDのチャンネルをお気に入りから外します

        Endpoint: `/api/channels/unfavorite`

        Parameters
        ----------
        channel_id : str | None
            対象のチャンネルID, default=None

        Returns
        -------
        bool
            お気に入りから外せたかどうか
        """
        channel_id = channel_id or self._channel_id

        return await super().unfavorite(channel_id=channel_id)


class ChannelActions(SharedChannelActions):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        super().__init__(session=session, client=client)

    async def create(
        self,
        name: str,
        color: str = MISSING,
        description: str = MISSING,
        banner_id: str = MISSING,
        is_sensitive: bool = MISSING,
        allow_renote_to_external: bool = MISSING,
    ) -> Channel:
        """チャンネルを作成します

        Endpoint: `/api/channels/create`

        Parameters
        ----------
        name : str
            チャンネル名
        color : str
            チャンネルの色, default=MISSING
        description : str
            チャンネルの説明, default=MISSING
        banner_id : str
            チャンネルのバナーに使用するファイルのID, default=MISSING
        is_sensitive : bool
            チャンネルがセンシティブかどうか, default=MISSING
        allow_renote_to_external : bool, optional
            外部へのリノートを許可するかどうか, default=MISSING

        Returns
        -------
        Channel
            作成したチャンネル
        """
        data = remove_dict_missing(
            {
                "name": name,
                "color": color,
                "description": description,
                "banner_id": banner_id,
                "is_sensitive": is_sensitive,
                "allow_renote_to_external": allow_renote_to_external,
            }
        )
        raw_channel: IChannel = await self._session.request(
            Route("POST", "/api/channels/create"),
            json=data,
            auth=True,
            lower=True,
            remove_none=False,
        )
        return Channel(raw_channel=raw_channel, client=self._client)

    async def featured(self) -> list[Channel]:
        """トレンドのチャンネルを取得します

        Endpoint: `/api/channels/featured`

        Returns
        -------
        list[Channel]
            取得したチャンネルのリスト
        """

        raw_channels: list[IChannel] = await self._session.request(
            Route("POST", "/api/channels/featured"), auth=True, lower=True
        )
        return [
            Channel(raw_channel=raw_channel, client=self._client) for raw_channel in raw_channels
        ]

    async def get_all_followed(
        self, since_id: str | None = None, until_id: str | None = None, limit: int = 5
    ) -> AsyncGenerator[Channel, None]:
        """フォロー中のすべてのチャンネルを取得します

        Endpoint: `/api/channels/followed`

        Parameters
        ----------
        since_id : str | None
            指定したチャンネルIDよりも後のチャンネルを取得します, default=None
        until_id : str | None
            指定したチャンネルIDよりも前のチャンネルを取得します, default=None
        limit : int, optional
            一度に取得するチャンネルの数, default=5

        Returns
        -------
        AsyncGenerator[Channel, None]
            取得したフォロー中のチャンネル
        """

        body = {"sinceId": since_id, "untilId": until_id, "limit": limit}

        pagination = Pagination[IChannel](
            self._session, Route("POST", "/api/channels/followed"), json=body, auth=True
        )

        while pagination.is_final is False:
            for raw_channel in await pagination.next():
                yield Channel(raw_channel=raw_channel, client=self._client)

    async def owned(
        self, since_id: str | None = None, until_id: str | None = None, limit: int = 5
    ) -> list[Channel]:
        """自分が所有しているチャンネル一覧を取得します

        Endpoint: `/api/channels/owned`

        Parameters
        ----------
        since_id : str, optional
            指定したチャンネルIDよりも後のチャンネルを取得します, default=None
        until_id : str, optional
            指定したチャンネルIDよりも前のチャンネルを取得します, default=None
        limit : int, optional
            一度に取得するチャンネルの数, default=5

        Returns
        -------
        list[Channel]
            取得した自分が所有しているチャンネルのリスト
        """
        data = {"sinceId": since_id, "untilId": until_id, "limit": limit}

        raw_channels: list[IChannel] = await self._session.request(
            Route("POST", "/api/channels/owned"), auth=True, json=data
        )
        return [
            Channel(raw_channel=raw_channel, client=self._client) for raw_channel in raw_channels
        ]

    async def get_all_owned(
        self, since_id: str | None = None, until_id: str | None = None, limit: int = 5
    ) -> AsyncGenerator[Channel, None]:
        """自分が所有しているすべてのチャンネルを取得します

        Endpoint: `/api/channels/owned`

        Parameters
        ----------
        since_id : str, optional
            指定したチャンネルIDよりも後のチャンネルを取得します, default=None
        until_id : str, optional
            指定したチャンネルIDよりも前のチャンネルを取得します, default=None
        limit : int, optional
            一度に取得するチャンネルの数, default=5

        Returns
        -------
        AsyncGenerator[Channel, None]
            取得した自分が所有しているチャンネル
        """

        body = {"sinceId": since_id, "untilId": until_id, "limit": limit}

        pagination = Pagination[IChannel](
            self._session, Route("POST", "/api/channels/owned"), json=body, auth=True
        )

        while pagination.is_final is False:
            for raw_channel in await pagination.next():
                yield Channel(raw_channel=raw_channel, client=self._client)

    async def show(self, channel_id: str) -> Channel:
        """指定したIDのチャンネルを取得します

        Endpoint: `/api/channels/show`

        Parameters
        ----------
        channel_id : str
            対象のチャンネルID

        Returns
        -------
        Channel
            取得したチャンネル
        """
        raw_channel: IChannel = await self._session.request(
            Route("POST", "/api/channels/show"), auth=True, json={"channelId": channel_id}
        )
        return Channel(raw_channel=raw_channel, client=self._client)

    async def my_favorites(self) -> list[Channel]:
        """自分がお気に入りにしているチャンネルを取得します

        Endpoint: `/api/channels/myFavorites`

        Returns
        -------
        list[Channel]
            取得したチャンネルのリスト
        """
        raw_channels: list[IChannel] = await self._session.request(
            Route("POST", "/api/channels/my-favorites"), auth=True, lower=True
        )
        return [
            Channel(raw_channel=raw_channel, client=self._client) for raw_channel in raw_channels
        ]

    async def search(
        self,
        query: str,
        type: Literal["nameAndDescription", "nameOnly"] = "nameAndDescription",
        since_id: str | None = None,
        until_id: str | None = None,
        limit: int = 5,
    ) -> list[Channel]:
        """チャンネルを検索します

        Endpoint: `/api/channels/search`

        Parameters
        ----------
        query : str
            検索するキーワード
        type : Literal["nameAndDescription","nameOnly"]
            検索に用いる形式, default="nameAndDescription"
        since_id : str | None
            指定したIDのチャンネルより後のチャンネルを取得します, default=None
        until_id : str, optional
            指定したIDのチャンネルより前のチャンネルを取得します, default=None
        limit : int, optional
            一度に取得するチャンネルの数, default=5

        Returns
        -------
        list[Channel]
            見つかったチャンネルのリスト
        """

        data = {
            "query": query,
            "type": type,
            "sinceId": since_id,
            "untilId": until_id,
            "limit": limit,
        }

        raw_channels: list[IChannel] = await self._session.request(
            Route("POST", "/api/channels/search"), auth=True, json=data
        )
        return [
            Channel(raw_channel=raw_channel, client=self._client) for raw_channel in raw_channels
        ]



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.actions.client import ClientActions
from mipac.http import HTTPClient
from mipac.manager.admins.admin import AdminManager
from mipac.manager.antenna import AntennaManager
from mipac.manager.channel import ChannelManager, ClientChannelManager
from mipac.manager.chart import ChartManager
from mipac.manager.clip import ClientClipManager, ClipManager
from mipac.manager.drive.drive import DriveManager
from mipac.manager.emoji import EmojiManager
from mipac.manager.follow import FollowManager, FollowRequestManager
from mipac.manager.invite import ClientInviteManager, InviteManager
from mipac.manager.my import MyManager
from mipac.manager.note import ClientNoteManager, NoteManager
from mipac.manager.role import RoleManager
from mipac.manager.user import ClientUserManager, UserManager
from mipac.manager.username import UsernameManager

if TYPE_CHECKING:
    from mipac.config import Config
    from mipac.models.lite.user import PartialUser
    from mipac.models.user import MeDetailed


__all__ = ("ClientManager",)


class ClientManager:
    def __init__(self, session: HTTPClient, config: Config):
        self.__session: HTTPClient = session
        self.i = MyManager(session=session, client=self)
        self.note: NoteManager = NoteManager(session=session, client=self)
        self.user: UserManager = UserManager(session=session, client=self)
        self.admin: AdminManager = AdminManager(session=session, client=self)
        self.drive: DriveManager = DriveManager(session=session, client=self)
        self.chart: ChartManager = ChartManager(session=session, client=self)
        self.channel: ChannelManager = ChannelManager(session=session, client=self)
        self.follow: FollowManager = FollowManager(
            session=session,
            client=self,
        )
        self.follow_request: FollowRequestManager = FollowRequestManager(
            session=session,
            client=self,
        )
        self.clip: ClipManager = ClipManager(session=session, client=self)
        self.emoji: EmojiManager = EmojiManager(session=session, client=self)
        self.antenna: AntennaManager = AntennaManager(session=session, client=self)
        self.role: RoleManager = RoleManager(session=session, client=self)
        self.username: UsernameManager = UsernameManager(session=session, client=self)
        self.invite: InviteManager = InviteManager(session=session, client=self)
        self._config: Config = config

    @property
    def action(self) -> ClientActions:
        return ClientActions(session=self.__session, client=self)

    def _create_user_instance(self, user: PartialUser) -> UserManager:
        return UserManager(session=self.__session, client=self)

    def _create_client_channel_manager(self, channel_id: str) -> ClientChannelManager:
        return ClientChannelManager(channel_id=channel_id, session=self.__session, client=self)

    def _create_client_note_manager(self, note_id: str) -> ClientNoteManager:
        """Create a client note manager.

        Returns
        -------
        ClientNoteManager
            The client note manager.
        """
        return ClientNoteManager(note_id=note_id, session=self.__session, client=self)

    def _create_client_invite_manager(self, invite_id: str) -> ClientInviteManager:
        return ClientInviteManager(invite_id=invite_id, session=self.__session, client=self)

    def _create_client_user_manager(self, user: PartialUser) -> ClientUserManager:
        return ClientUserManager(user=user, session=self.__session, client=self)

    def _get_client_clip_instance(self, *, clip_id: str) -> ClientClipManager:
        return ClientClipManager(clip_id=clip_id, session=self.__session, client=self)

    async def get_me(self) -> MeDetailed:
        return await self.user.action.get_me()



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.manager import AbstractManager
from mipac.actions.invite import ClientInviteActions, InviteActions
from mipac.http import HTTPClient

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class ClientInviteManager(AbstractManager):
    def __init__(self, invite_id: str, *, session: HTTPClient, client: ClientManager):
        self.__session: HTTPClient = session
        self.__client: ClientManager = client
        self.__actions: ClientInviteActions = ClientInviteActions(
            invite_id=invite_id, session=session, client=client
        )

    @property
    def action(self) -> ClientInviteActions:
        return self.__actions


class InviteManager(AbstractManager):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        self.__session: HTTPClient = session
        self.__client: ClientManager = client
        self.__actions: InviteActions = InviteActions(session=session, client=client)

    @property
    def action(self) -> InviteActions:
        return self.__actions



from __future__ import annotations

from datetime import datetime
from typing import TYPE_CHECKING, Any

from mipac.models.user import UserDetailedNotMe, packed_user
from mipac.types.blocking import IBlocking
from mipac.utils.format import str_to_datetime

if TYPE_CHECKING:
    from mipac.manager.blocking import ClientBlockingManager
    from mipac.manager.client import ClientManager


class Blocking:
    def __init__(self, *, raw_blocking: IBlocking, client: ClientManager) -> None:
        self.__raw_blocking: IBlocking = raw_blocking
        self.__client: ClientManager = client

    @property
    def id(self) -> str:
        """ブロックID

        Returns
        -------
        str
            ブロックID
        """
        return self.__raw_blocking["id"]

    @property
    def created_at(self) -> datetime:
        """ブロックした日時

        Returns
        -------
        datetime
            ブロックした日時
        """
        return str_to_datetime(self.__raw_blocking["created_at"])

    @property
    def blockee_id(self) -> str:
        """ブロック対象のユーザーID

        Returns
        -------
        str
            ブロック対象のユーザーID
        """
        return self.__raw_blocking["blockee_id"]

    @property
    def blockee(self) -> UserDetailedNotMe:
        """ブロック対象のユーザー情報

        Returns
        -------
        UserDetailedNotMe
            ブロック対象のユーザー情報
        """
        return packed_user(self.__raw_blocking["blockee"], client=self.__client)

    @property
    def api(self) -> ClientBlockingManager:
        """ブロック対象に対するAPIを利用するためのManager

        Returns
        -------
        ClientBlockingManager
            ブロック対象に対するAPIを利用するためのManager
        """
        return self.__client.user._create_client_blocking_manager(user_id=self.blockee.id)

    def _get(self, key: str) -> Any | None:
        return self.__raw_blocking.get(key)

    def __eq__(self, __value: object) -> bool:
        return isinstance(__value, Blocking) and self.id == __value.id

    def __ne__(self, __value: object) -> bool:
        return not self.__eq__(__value)



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from mipac.models.lite.ad import PartialAd
from mipac.types.meta import IPartialMeta, IPolicies

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class Policies:
    def __init__(self, raw_policies: IPolicies) -> None:
        self.__raw_policies: IPolicies = raw_policies

    @property
    def gtl_available(self) -> bool:
        """Whether GTL is effective"""
        return self.__raw_policies["gtl_available"]

    @property
    def ltl_available(self) -> bool:
        """Whether LTL is effective"""
        return self.__raw_policies["ltl_available"]

    @property
    def can_public_note(self) -> bool:
        """Whether you can post a public note"""
        return self.__raw_policies["can_public_note"]

    @property
    def can_edit_note(self) -> bool:
        """Whether you can edit a note"""
        return self.__raw_policies["can_edit_note"]

    @property
    def can_invite(self) -> bool:
        """Whether you can invite"""
        return self.__raw_policies["can_invite"]

    @property
    def invite_limit(self) -> int:
        return self.__raw_policies["invite_limit"]

    @property
    def invite_limit_cycle(self) -> int:
        return self.__raw_policies["invite_limit_cycle"]

    @property
    def invite_expiration_time(self) -> int:
        return self.__raw_policies["invite_expiration_time"]

    @property
    def can_manage_custom_emojis(self) -> bool:
        """Whether you can manage custom emojis"""
        return self.__raw_policies["can_manage_custom_emojis"]

    @property
    def can_search_notes(self) -> bool:
        """Whether you can search note"""
        return self.__raw_policies["can_search_notes"]

    @property
    def can_use_translator(self) -> bool:
        """Whether you can use translator"""
        return self.__raw_policies["can_use_translator"]

    @property
    def can_hide_ads(self) -> bool:
        """Whether you can hide ads"""
        return self.__raw_policies["can_hide_ads"]

    @property
    def drive_capacity_mb(self) -> int:
        return self.__raw_policies["drive_capacity_mb"]

    @property
    def always_mark_nsfw(self) -> bool:
        return self.__raw_policies["always_mark_nsfw"]

    @property
    def pin_limit(self) -> int:
        return self.__raw_policies["pin_limit"]

    @property
    def antenna_limit(self) -> int:
        return self.__raw_policies["antenna_limit"]

    @property
    def word_mute_limit(self) -> int:
        return self.__raw_policies["word_mute_limit"]

    @property
    def webhook_limit(self) -> int:
        return self.__raw_policies["webhook_limit"]

    @property
    def clip_limit(self) -> int:
        return self.__raw_policies["clip_limit"]

    @property
    def note_each_clips_limit(self) -> int:
        return self.__raw_policies["note_each_clips_limit"]

    @property
    def user_list_limit(self) -> int:
        return self.__raw_policies["user_list_limit"]

    @property
    def user_each_user_lists_limit(self) -> int:
        return self.__raw_policies["user_each_user_lists_limit"]

    @property
    def rate_limit_factor(self) -> int:
        return self.__raw_policies["rate_limit_factor"]

    def _get(self, key: str) -> Any | None:
        return self.__raw_policies.get(key)


class PartialMeta[T: IPartialMeta]:
    def __init__(self, raw_meta: T, *, client: ClientManager) -> None:
        self._raw_meta: T = raw_meta
        self.__client: ClientManager = client

    @property
    def maintainer_name(self) -> str | None:
        return self._raw_meta["maintainer_name"]

    @property
    def maintainer_email(self) -> str | None:
        return self._raw_meta["maintainer_email"]

    @property
    def version(self) -> str:
        return self._raw_meta["version"]

    @property
    def name(self) -> str | None:
        return self._raw_meta["name"]

    @property
    def short_name(self) -> str | None:
        return self._raw_meta["short_name"]

    @property
    def uri(self) -> str:
        return self._raw_meta["uri"]

    @property
    def description(self) -> str | None:
        return self._raw_meta["description"]

    @property
    def langs(self) -> list[str]:
        return self._raw_meta["langs"]

    @property
    def tos_url(self) -> str:
        return self._raw_meta["tos_url"]

    @property
    def repository_url(self) -> str:
        return self._raw_meta["repository_url"]

    @property
    def feedback_url(self) -> str:
        return self._raw_meta["feedback_url"]

    @property
    def disable_registration(self) -> bool:
        return self._raw_meta["disable_registration"]

    @property
    def email_required_for_signup(self) -> bool:
        return self._raw_meta["email_required_for_signup"]

    @property
    def enable_hcaptcha(self) -> bool:
        return self._raw_meta["enable_hcaptcha"]

    @property
    def hcaptcha_site_key(self) -> str | None:
        return self._raw_meta["hcaptcha_site_key"]

    @property
    def enable_recaptcha(self) -> bool:
        return self._raw_meta["enable_recaptcha"]

    @property
    def recaptcha_site_key(self) -> str:
        return self._raw_meta["recaptcha_site_key"]

    @property
    def enable_turnstile(self) -> bool:
        return self._raw_meta["enable_turnstile"]

    @property
    def turnstile_site_key(self) -> str:
        return self._raw_meta["turnstile_site_key"]

    @property
    def sw_publickey(self) -> str | None:
        return self._raw_meta["sw_publickey"]

    @property
    def theme_color(self) -> str:
        return self._raw_meta["theme_color"]

    @property
    def mascot_image_url(self) -> str:
        return self._raw_meta["mascot_image_url"]

    @property
    def banner_url(self) -> str | None:
        return self._raw_meta["banner_url"]

    @property
    def info_image_url(self) -> str | None:
        return self._raw_meta["info_image_url"]

    @property
    def server_error_image_url(self) -> str | None:
        return self._raw_meta["server_error_image_url"]

    @property
    def not_found_image_url(self) -> str | None:
        return self._raw_meta["not_found_image_url"]

    @property
    def icon_url(self) -> str | None:
        return self._raw_meta["icon_url"]

    @property
    def background_image_url(self) -> str | None:
        return self._raw_meta["background_image_url"]

    @property
    def logo_image_url(self) -> str | None:
        return self._raw_meta["logo_image_url"]

    @property
    def max_note_text_length(self) -> int:
        return self._raw_meta["max_note_text_length"]

    @property
    def default_light_theme(self) -> str | None:
        return self._raw_meta["default_light_theme"]

    @property
    def default_dark_theme(self) -> str | None:
        return self._raw_meta["default_dark_theme"]

    @property
    def ads(self) -> list[PartialAd]:
        return [PartialAd(raw_ad, client=self.__client) for raw_ad in self._raw_meta["ads"]]

    @property
    def enable_email(self) -> bool:
        return self._raw_meta["enable_email"]

    @property
    def enable_service_worker(self) -> bool:
        return self._raw_meta["enable_service_worker"]

    @property
    def translator_available(self) -> bool:
        return self._raw_meta["translator_available"]

    @property
    def server_rules(self) -> list[str]:
        return self._raw_meta["server_rules"]

    @property
    def policies(self) -> IPolicies:
        return self._raw_meta["policies"]

    @property
    def media_proxy(self) -> str:
        return self._raw_meta["media_proxy"]

    def _get(self, key: str) -> Any | None:
        return self._raw_meta.get(key)



from __future__ import annotations

from typing import TYPE_CHECKING, Literal, NotRequired, TypedDict, TypeGuard

from mipac.types.meta import IPolicies
from mipac.types.roles import IPartialRole

if TYPE_CHECKING:
    from mipac.types.announcement import IAnnouncement
    from mipac.types.instance import IInstanceLite
    from mipac.types.note import INote
    from mipac.types.page import IPage

IUserOnlineStatus = Literal["online", "active", "offline", "unknown"]
IFfVisibility = Literal["public", "followers", "private"]
IUserNotify = Literal["normal", "none"]
ITwoFactorBackupCodesStock = Literal["full", "partial", "none"]
NotificationRecieveConfigOption = Literal[
    "all", "following", "follower", "mutualFollow", "never"
]  # Misskey側が間違っている(Receiveのミススペル?)ので混乱を招かないようにこっちも統一してある
EmailNotificationTypes = Literal["mention", "reply", "quote", "follow", "receiveFollowRequest"]


class IUserField(TypedDict):
    name: str
    value: str


class ISignin(TypedDict):
    id: str
    created_at: str
    user_id: str
    ip: str
    headers: dict[str, str]
    success: bool


class IAchievement(TypedDict):
    name: str
    unlocked_at: int


class IBadgeRole(TypedDict):
    name: str
    icon_url: str | None
    display_order: int


class IUserRole(IBadgeRole):
    """
    Deprecated

    Will be removed in v0.7.0
    """

    id: str
    color: str | None
    description: str
    is_moderator: bool
    is_administrator: bool


class IUserRequired(TypedDict):
    id: str
    username: str
    name: str
    online_status: Literal["online", "active", "offline", "unknown"]
    avatar_url: str
    avatar_blurhash: str


class IAvatarDecoration(TypedDict):
    id: str
    angle: NotRequired[int]
    flip_h: NotRequired[bool]
    url: str
    offset_x: NotRequired[int]
    offset_y: NotRequired[int]


class NotificationRecieveConfigType(TypedDict):
    type: NotificationRecieveConfigOption


class NotificationRecieveConfigWithUserList(TypedDict):
    type: NotificationRecieveConfigOption
    user_list_id: str


class NotificationRecieveConfig(TypedDict):
    note: NotificationRecieveConfigType | NotificationRecieveConfigWithUserList
    follow: NotificationRecieveConfigType | NotificationRecieveConfigWithUserList
    mention: NotificationRecieveConfigType | NotificationRecieveConfigWithUserList
    reply: NotificationRecieveConfigType | NotificationRecieveConfigWithUserList
    renote: NotificationRecieveConfigType | NotificationRecieveConfigWithUserList
    quote: NotificationRecieveConfigType | NotificationRecieveConfigWithUserList
    reaction: NotificationRecieveConfigType | NotificationRecieveConfigWithUserList
    pollEnded: NotificationRecieveConfigType | NotificationRecieveConfigWithUserList
    receive_follow_request: NotificationRecieveConfigType | NotificationRecieveConfigWithUserList
    follow_request_accepted: NotificationRecieveConfigType | NotificationRecieveConfigWithUserList
    role_assigned: NotificationRecieveConfigType | NotificationRecieveConfigWithUserList
    achievement_earned: NotificationRecieveConfigType | NotificationRecieveConfigWithUserList
    app: NotificationRecieveConfigType | NotificationRecieveConfigWithUserList
    test: NotificationRecieveConfigType | NotificationRecieveConfigWithUserList


class IUserSecurityKey(TypedDict):
    id: str
    name: str
    last_used: str


class IPartialUser(TypedDict):
    """
    Misskey Schema: `packedUserLiteSchema`
    """

    id: str
    name: str | None
    username: str
    host: str | None
    avatar_url: str | None
    avatar_blurhash: str | None
    avatar_decorations: list[IAvatarDecoration]
    is_bot: NotRequired[bool]
    is_cat: NotRequired[bool]
    instance: NotRequired[IInstanceLite]  # ローカルユーザーの場合はキーが無い
    emojis: dict[str, str]
    online_status: IUserOnlineStatus
    badge_roles: NotRequired[list[IBadgeRole]]  # リモートユーザーの場合はキーが無い


class IUserDetailedNotMeOnlySchema(TypedDict):
    """
    Misskey Schema: `packedUserDetailedNotMeOnlySchema`
    """

    url: str | None
    uri: str | None
    moved_to: str | None
    also_known_as: list[str] | None
    created_at: str
    updated_at: str | None
    last_fetched_at: str | None
    banner_url: str | None
    banner_blurhash: str | None
    is_locked: bool
    is_silenced: bool
    is_suspended: bool
    description: str | None
    location: str | None
    birthday: str | None
    lang: str | None
    fields: list[IUserField]
    verified_links: list[str]
    followers_count: int
    following_count: int
    notes_count: int
    pinned_note_ids: list[str]
    pinned_notes: list[INote]
    pinned_page_id: str | None
    pinned_page: IPage | None  # TODO: IPageが正しいか確認する
    public_reactions: bool
    following_visibility: IFfVisibility
    followers_visibility: IFfVisibility
    two_factor_enabled: bool
    use_password_less_login: bool
    security_keys: bool
    roles: list[IPartialRole]
    memo: str | None
    moderation_note: NotRequired[str]
    is_following: NotRequired[bool]
    is_followed: NotRequired[bool]
    has_pending_follow_request_from_you: NotRequired[bool]
    has_pending_follow_request_to_you: NotRequired[bool]
    is_blocking: NotRequired[bool]
    is_blocked: NotRequired[bool]
    is_muted: NotRequired[bool]
    is_renote_muted: NotRequired[bool]
    notify: NotRequired[IUserNotify]
    with_replies: NotRequired[bool]


class IMeDetailedOnlySchema(TypedDict):
    avatar_id: str | None
    banner_id: str | None
    is_moderator: bool | None  # entities/UserEntityService.ts で roleServiceを用いて判断してるからNoneの場合がある?
    is_admin: bool | None
    inject_featured_note: bool
    receive_announcement_email: bool
    always_mark_nsfw: bool
    auto_sensitive: bool
    careful_bot: bool
    auto_accept_followed: bool
    no_crawle: bool
    prevent_ai_learning: bool
    is_explorable: bool
    is_deleted: bool
    two_factor_backup_codes_stock: ITwoFactorBackupCodesStock
    hide_online_status: bool
    has_unread_specified_notes: bool
    has_unread_mentions: bool
    has_unread_announcement: bool
    unread_announcements: list[IAnnouncement]
    has_unread_antenna: bool
    has_unread_channel: bool
    has_unread_notification: bool
    has_pending_received_follow_request: bool
    unread_notifications_count: bool
    muted_words: list[list[str]]
    hard_muted_words: list[list[str]]
    muted_instances: list[str]
    notification_recieve_config: NotificationRecieveConfig
    email_notification_types: list[EmailNotificationTypes]
    achievements: list[IAchievement]
    logged_in_days: int
    policies: IPolicies
    email: NotRequired[str | None]
    email_verified: NotRequired[bool]
    security_keys_list: NotRequired[list[IUserSecurityKey]]  # セキュリティー


class IUserList(TypedDict):
    id: str
    created_at: str
    name: str
    user_ids: list[str]
    is_public: bool


class IUserListMembership(TypedDict):
    id: str
    created_at: str
    user_id: str
    user: IPartialUser
    with_replies: bool


class IUserDetailedNotMeSchema(IPartialUser, IUserDetailedNotMeOnlySchema):
    pass


class IMeDetailedSchema(IUserDetailedNotMeSchema, IMeDetailedOnlySchema):
    pass


IUserDetailed = IUserDetailedNotMeSchema | IMeDetailedSchema

IUser = IPartialUser | IUserDetailed | IUserDetailedNotMeSchema


def is_partial_user(user: IUser) -> TypeGuard[IPartialUser]:
    """
    他のUser型は全て IUserDetailedNotMeSchema 経由で IUserDetailedNotMeOnlySchema を継承しているため
    url が無いことを確認し区別する
    念のために avatar_url があることも確認する
    """
    if "url" not in user and "avatar_url" in user:
        return True
    return False


def is_user_detailed_not_me(user: IUser) -> TypeGuard[IUserDetailedNotMeSchema]:
    """

    IUserDetailedNotMeSchemaが持つ url が有ることを確認し
    IMeDetailedOnlySchema が持つ avatar_id が無いことを確認する
    こうすることで IMeDetailedSchema と区別する
    """
    if "avatar_id" not in user and "url" in user:
        return True
    return False


def is_me_detailed(user: IUser) -> TypeGuard[IMeDetailedSchema]:
    """
    IMeDetailedOnlySchemaで avatar_id
    IUserDetailedNotMeOnlySchemaで url
    を持っているのでどちらともを満たしたものがIMeDetailedSchema
    """
    if "avatar_id" in user and "url" in user:
        return True
    return False


def is_user_detailed(user: IUser) -> TypeGuard[IUserDetailed]:
    if is_user_detailed_not_me(user) or is_me_detailed(user):
        return True
    return False


####
##  ここからモデルというよりレスポンス
####


class GetFrequentlyRepliedUsersResponse(TypedDict):
    """`users/get-frequently-replied-users` のレスポンス"""

    user: IUserDetailed
    weight: int



from __future__ import annotations
from typing import Any

from mipac.types.instance import IInstanceLite


class LiteInstance:
    def __init__(self, instance: IInstanceLite) -> None:
        self.__instance: IInstanceLite = instance

    @property
    def name(self) -> str | None:
        return self.__instance["name"]

    @property
    def software_name(self) -> str | None:
        return self.__instance["software_name"]

    @property
    def software_version(self) -> str | None:
        return self.__instance["software_version"]

    @property
    def icon_url(self) -> str | None:
        return self.__instance["icon_url"]

    @property
    def favicon_url(self) -> str | None:
        return self.__instance["favicon_url"]

    @property
    def theme_color(self) -> str | None:
        return self.__instance["theme_color"]

    def _get(self, key: str) -> Any | None:
        return self.__instance.get(key)



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.manager import AbstractManager
from mipac.actions.users.list import (
    ClientPartialUserListActions,
    ClientUserListActions,
    UserListActions,
)
from mipac.http import HTTPClient

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class ClientPartialUserListManager(AbstractManager):
    def __init__(self, user_id: str, *, session: HTTPClient, client: ClientManager):
        self.__user_id: str = user_id
        self.__session: HTTPClient = session
        self.__client: ClientManager = client
        self.__action: ClientPartialUserListActions = ClientPartialUserListActions(
            user_id=self.__user_id, session=self.__session, client=self.__client
        )

    @property
    def action(self) -> ClientPartialUserListActions:
        return self.__action


class ClientUserListManager(AbstractManager):
    def __init__(
        self,
        list_id: str,
        *,
        session: HTTPClient,
        client: ClientManager,
    ):
        self.__session: HTTPClient = session
        self.__client: ClientManager = client
        self.__action: ClientUserListActions = ClientUserListActions(
            list_id=list_id, session=self.__session, client=self.__client
        )

    @property
    def action(self) -> ClientUserListActions:
        return self.__action


class UserListManager(AbstractManager):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        self.__session: HTTPClient = session
        self.__client: ClientManager = client
        self.__action: UserListActions = UserListActions(
            session=self.__session, client=self.__client
        )

    @property
    def action(self) -> UserListActions:
        return self.__action



from __future__ import annotations

from typing import TYPE_CHECKING, AsyncGenerator, Literal, override

from mipac.abstract.action import AbstractAction
from mipac.http import HTTPClient, Route
from mipac.models.ad import Ad
from mipac.types.ads import IAd
from mipac.utils.pagination import Pagination

if TYPE_CHECKING:
    from mipac.client import ClientManager


class SharedAdminAdActions(AbstractAction):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        self._session: HTTPClient = session
        self._client: ClientManager = client

    async def delete(self, *, ad_id: str) -> bool:
        res: bool = await self._session.request(
            Route("POST", "/api/admin/ad/delete"), json={"id": ad_id}, auth=True, lower=True
        )
        return res

    async def update(
        self,
        memo: str,
        url: str,
        image_url: str,
        place: Literal["square", "horizontal", "horizontal-big"],
        priority: Literal["high", "middle", "low"],
        ratio: int,
        expires_at: int,
        starts_at: int,
        day_of_week: int,
        *,
        ad_id: str,
    ) -> bool:
        data = {
            "id": ad_id,
            "memo": memo or "",
            "url": url,
            "imageUrl": image_url,
            "place": place,
            "priority": priority,
            "ratio": ratio,
            "expiresAt": expires_at,
            "startsAt": starts_at,
            "dayOfWeek": day_of_week,
        }
        res: bool = await self._session.request(
            Route("POST", "/api/admin/ad/update"), json=data, auth=True, lower=True
        )
        return res


class ClientAdminAdActions(SharedAdminAdActions):
    def __init__(self, ad_id: str, *, session: HTTPClient, client: ClientManager):
        super().__init__(session=session, client=client)
        self._ad_id: str = ad_id

    @override
    async def delete(self, *, ad_id: str | None = None) -> bool:
        ad_id = ad_id or self._ad_id

        return await super().delete(ad_id=ad_id)

    @override
    async def update(
        self,
        memo: str,
        url: str,
        image_url: str,
        place: Literal["square", "horizontal", "horizontal-big"],
        priority: Literal["high", "middle", "low"],
        ratio: int,
        expires_at: int,
        starts_at: int,
        day_of_week: int,
        *,
        ad_id: str | None = None,
    ) -> bool:
        ad_id = ad_id or self._ad_id

        return await super().update(
            memo=memo,
            url=url,
            image_url=image_url,
            place=place,
            priority=priority,
            ratio=ratio,
            expires_at=expires_at,
            starts_at=starts_at,
            day_of_week=day_of_week,
            ad_id=ad_id,
        )


class AdminAdActions(SharedAdminAdActions):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        super().__init__(session=session, client=client)

    async def create(
        self,
        url: str,
        memo: str,
        place: Literal["square", "horizontal", "horizontal-big"],
        priority: Literal["high", "middle", "low"],
        ratio: int,
        expires_at: str,
        starts_at: str,
        image_url: str,
        day_of_week: int,
    ) -> Ad:
        data = {
            "url": url,
            "memo": memo or "",
            "place": place,
            "priority": priority,
            "ratio": ratio,
            "expiresAt": expires_at,
            "startsAt": starts_at,
            "imageUrl": image_url,
            "dayOfWeek": day_of_week,
        }
        raw_ad: IAd = await self._session.request(
            Route("POST", "/api/admin/ad/create"), json=data, auth=True, lower=True
        )
        return Ad(ad_data=raw_ad, client=self._client)

    async def get_list(
        self,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
        publishing: bool | None = None,
    ):
        data = {
            "limit": limit,
            "sinceId": since_id,
            "until_id": until_id,
            "publishing": publishing,
        }

        raw_ads: list[IAd] = await self._session.request(
            Route("POST", "/api/admin/ad/list"), auth=True, json=data
        )

        return [Ad(raw_ad, client=self._client) for raw_ad in raw_ads]

    async def get_all_list(
        self,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
        publishing: bool | None = None,
    ) -> AsyncGenerator[Ad, None]:
        data = {
            "limit": limit,
            "sinceId": since_id,
            "until_id": until_id,
            "publishing": publishing,
        }

        pagination = Pagination[IAd](
            http_client=self._session, route=Route("POST", "/api/admin/ad/list"), json=data
        )

        while pagination.is_final is False:
            raw_ads = await pagination.next()

            for raw_ad in raw_ads:
                yield Ad(ad_data=raw_ad, client=self._client)



from typing import Literal

Permissions = Literal[
    "read:account",
    "write:account",
    "read:blocks",
    "write:blocks",
    "read:drive",
    "write:drive",
    "read:favorites",
    "write:favorites",
    "read:following",
    "write:following",
    "read:messaging",
    "write:messaging",
    "read:mutes",
    "write:mutes",
    "write:notes",
    "read:notifications",
    "write:notifications",
    "read:reactions",
    "write:reactions",
    "write:votes",
    "read:pages",
    "write:pages",
    "write:page-likes",
    "read:page-likes",
    "read:user-groups",
    "write:user-groups",
    "read:channels",
    "write:channels",
    "read:gallery",
    "write:gallery",
    "read:gallery-likes",
    "write:gallery-likes",
    "read:flash",
    "write:flash",
    "read:flash-likes",
    "write:flash-likes",
    "read:admin:abuse-user-reports",
    "write:admin:delete-account",
    "write:admin:delete-all-files-of-a-user",
    "read:admin:index-stats",
    "read:admin:table-stats",
    "read:admin:user-ips",
    "read:admin:meta",
    "write:admin:reset-password",
    "write:admin:resolve-abuse-user-report",
    "write:admin:send-email",
    "read:admin:server-info",
    "read:admin:show-moderation-log",
    "read:admin:show-user",
    "read:admin:show-users",
    "write:admin:suspend-user",
    "write:admin:unset-user-avatar",
    "write:admin:unset-user-banner",
    "write:admin:unsuspend-user",
    "write:admin:meta",
    "write:admin:user-note",
    "write:admin:roles",
    "read:admin:roles",
    "write:admin:relays",
    "read:admin:relays",
    "write:admin:invite-codes",
    "read:admin:invite-codes",
    "write:admin:announcements",
    "read:admin:announcements",
    "write:admin:avatar-decorations",
    "read:admin:avatar-decorations",
    "write:admin:federation",
    "write:admin:account",
    "read:admin:account",
    "write:admin:emoji",
    "read:admin:emoji",
    "write:admin:queue",
    "read:admin:queue",
    "write:admin:promo",
    "write:admin:drive",
    "read:admin:drive",
    "write:admin:ad",
    "read:admin:ad",
    "write:invite-codes",
    "read:invite-codes",
    "write:clip-favorite",
    "read:clip-favorite",
    "read:federation",
    "write:report-abuse",
]



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.manager import AbstractManager
from mipac.actions.drive.drive import DriveActions
from mipac.http import HTTPClient
from mipac.manager.drive.files import ClientFileManager, DriveFileManager
from mipac.manager.drive.folders import ClientFolderManager, FolderManager

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class DriveManager(AbstractManager):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        self.__session: HTTPClient = session
        self.__client: ClientManager = client
        self.__action: DriveActions = DriveActions(session=session, client=client)
        self.files: DriveFileManager = DriveFileManager(session=session, client=client)
        self.folders: FolderManager = FolderManager(session=session, client=client)

    @property
    def action(self) -> DriveActions:
        return self.__action

    def _create_client_file_manager(self, *, file_id: str) -> ClientFileManager:
        return ClientFileManager(file_id=file_id, session=self.__session, client=self.__client)

    def _create_client_folder_manager(self, *, folder_id: str) -> ClientFolderManager:
        return ClientFolderManager(
            folder_id=folder_id, session=self.__session, client=self.__client
        )



from __future__ import annotations

from typing import TYPE_CHECKING, override

from mipac.abstract.action import AbstractAction
from mipac.http import HTTPClient, Route
from mipac.utils.util import deprecated

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class SharedAdminUserActions(AbstractAction):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        self._session = session
        self._client = client

    @deprecated
    async def delete_account(self, *, user_id: str) -> bool:
        """対象のユーザーを削除します

        Endpoint: `/api/admin/accounts/delete`

        .. deprecated:: 0.6.2
            Use :meth:`mipac.actions.admins.accounts.delete` instead.

        Parameters
        ----------
        user_id : str
            対象のユーザーID

        Returns
        -------
        bool
            成功ならTrue
        """
        return await self._client.admin.account.action.delete(user_id=user_id)

    async def show_user(self, *, user_id: str):  # TODO: 専用の型が必要
        """ユーザーの情報を取得します

        Parameters
        ----------
        user_id : str
            対象のユーザーID
        """
        data = {"userId": user_id}

        raw_user = await self._session.request(
            Route("GET", "/api/admin/show-user"),
            json=data,
            auth=True,
            lower=True,
        )
        return raw_user

    async def suspend(self, *, user_id: str) -> bool:
        """対象のユーザーを凍結します

        Parameters
        ----------
        user_id : str
            対象のユーザーID

        Returns
        -------
        bool
            成功ならTrue
        """
        data = {"userId": user_id}

        res: bool = await self._session.request(
            Route("POST", "/api/admin/suspend-user"),
            json=data,
            auth=True,
            lower=True,
        )
        return res

    async def unsuspend(self, *, user_id: str) -> bool:
        """ユーザーの凍結を解除します

        Parameters
        ----------
        user_id : str
            対象のユーザーID

        Returns
        -------
        bool
            成功ならTrue
        """
        data = {"userId": user_id}

        res: bool = await self._session.request(
            Route("POST", "/api/admin/unsuspend-user"),
            json=data,
            auth=True,
            lower=True,
        )
        return res


class ClientAdminUserActions(SharedAdminUserActions):
    def __init__(self, user_id: str, *, session: HTTPClient, client: ClientManager):
        super().__init__(session=session, client=client)
        self.__user_id = user_id

    @override
    async def delete_account(self, *, user_id: str | None = None) -> bool:
        user_id = user_id or self.__user_id

        return await super().delete_account(user_id=user_id)

    @override
    async def show_user(self, *, user_id: str | None = None):
        """ユーザーの情報を取得します

        Parameters
        ----------
        user_id : str
            対象のユーザーID
        """
        user_id = user_id or self.__user_id

        return await super().show_user(user_id=user_id)

    @override
    async def suspend(self, *, user_id: str | None = None) -> bool:
        """対象のユーザーを凍結します

        Parameters
        ----------
        user_id : str
            対象のユーザーID

        Returns
        -------
        bool
            成功ならTrue
        """

        user_id = user_id or self.__user_id

        return await super().suspend(user_id=user_id)

    @override
    async def unsuspend(self, *, user_id: str | None = None) -> bool:
        """ユーザーの凍結を解除します

        Parameters
        ----------
        user_id : str
            対象のユーザーID

        Returns
        -------
        bool
            成功ならTrue
        """
        user_id = user_id or self.__user_id

        return await super().unsuspend(user_id=user_id)


class AdminUserActions(SharedAdminUserActions):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        super().__init__(session=session, client=client)



from __future__ import annotations

from datetime import datetime
from typing import TYPE_CHECKING, Any

from mipac.models.drive import File
from mipac.models.lite.user import PartialUser
from mipac.types.gallery import IGalleryPost
from mipac.utils.format import str_to_datetime

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class GalleryPost:
    def __init__(self, raw_gallery: IGalleryPost, *, client: ClientManager) -> None:
        self._raw_gallery: IGalleryPost = raw_gallery
        self.__client: ClientManager = client

    @property
    def id(self) -> str:
        return self._raw_gallery["id"]

    @property
    def created_at(self) -> datetime:
        return str_to_datetime(self._raw_gallery["created_at"])

    @property
    def updated_at(self) -> datetime:
        return str_to_datetime(self._raw_gallery["updated_at"])

    @property
    def user_id(self) -> str:
        return self._raw_gallery["user_id"]

    @property
    def user(self) -> PartialUser:
        return PartialUser(self._raw_gallery["user"], client=self.__client)

    @property
    def title(self) -> str:
        return self._raw_gallery["title"]

    @property
    def description(self) -> str | None:
        return self._raw_gallery["description"]

    @property
    def file_ids(self) -> list[str]:
        return self._raw_gallery["file_ids"]

    @property
    def files(self) -> list[File]:
        return [File(file, client=self.__client) for file in self._raw_gallery["files"]]

    @property
    def tags(self) -> list[str] | None:
        return self._raw_gallery.get("tags")

    @property
    def is_sensitive(self) -> bool:
        return self._raw_gallery["is_sensitive"]

    @property
    def liked_count(self) -> int:
        return self._raw_gallery["liked_count"]

    @property
    def is_liked(self) -> bool | None:
        return self._raw_gallery.get("is_liked")

    def _get(self, key: str) -> Any | None:
        return self._raw_gallery.get(key)



from __future__ import annotations

from typing import TYPE_CHECKING, override

from mipac.abstract.action import AbstractAction
from mipac.http import HTTPClient, Route
from mipac.models.drive import File, Folder
from mipac.types.drive import IFolder
from mipac.utils.format import remove_dict_missing
from mipac.utils.util import MISSING

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class ClientFileActionsInFolder(AbstractAction):
    """File actions in a folder"""

    def __init__(self, folder_id: str, *, session: HTTPClient, client: ClientManager):
        self._folder_id: str = folder_id
        self._session: HTTPClient = session
        self._client: ClientManager = client

    async def create(
        self,
        file: str,
        name: str | None = None,
        comment: str | None = None,
        is_sensitive: bool = False,
        force: bool = False,
    ) -> File:
        """Upload a file to the drive

        Endpoint: `/api/drive/files/create`

        Parameters
        ----------
        file: str
            The file to upload
        folder_id: str | None
            The id of the folder to upload the file to, defaults to None
        name: str | None
            The name of the file, defaults to None
        comment: str | None
            The comment of the file, defaults to None
        is_sensitive: bool
            Whether the file is sensitive or not, defaults to False
        force: bool
            Whether to force upload the file or not, defaults to False

        Returns
        -------
        File
            The uploaded file
        """
        return await self._client.drive.files.action.create(
            file=file,
            name=name,
            comment=comment,
            is_sensitive=is_sensitive,
            force=force,
            folder_id=self._folder_id,
        )

    async def update(
        self,
        file_id: str,
        name: str | None = MISSING,
        is_sensitive: bool = MISSING,
        comment: str | None = MISSING,
    ):
        """Update a file

        Endpoint: `/api/drive/files/update`

        Parameters
        ----------
        file_id: str | None
            The id of the file to update, defaults to None
        name: str | None
            The name of the file, defaults to MISSING
        is_sensitive: bool
            Whether the file is sensitive or not, defaults to MISSING
        comment: str | None
            The comment of the file, defaults to MISSING

        Returns
        -------
        File
            The updated file
        """
        return await self._client.drive.files.action.update(
            file_id=file_id,
            name=name,
            is_sensitive=is_sensitive,
            comment=comment,
            folder_id=self._folder_id,
        )

    async def find(self, name: str) -> list[File]:
        """Find a file by its name

        Endpoint: `/api/drive/files/find`

        Parameters
        ----------
        name: str
            The name of the file to find

        Returns
        -------
        list[File]
            The found files
        """
        return await self._client.drive.files.action.find(name=name, folder_id=self._folder_id)

    async def upload_from_url(
        self,
        url: str,
        is_sensitive: bool = False,
        comment: str | None = None,
        marker: str | None = None,
        force: bool = False,
    ):
        """Upload a file to the drive from a url

        Endpoint: `/api/drive/files/upload-from-url`

        Parameters
        ----------
        url: str
            The url of the file to upload
        is_sensitive: bool
            Whether the file is sensitive or not, defaults to False
        comment: str | None
            The comment of the file, defaults to None
        marker: str | None
            The marker of the file, defaults to None
        force: bool
            Whether to force upload the file or not, defaults to False

        Returns
        -------
        bool
            Whether the file was uploaded or not
        """
        return await self._client.drive.files.action.upload_from_url(
            url=url,
            folder_id=self._folder_id,
            is_sensitive=is_sensitive,
            comment=comment,
            marker=marker,
            force=force,
        )


class SharedFolderActions(AbstractAction):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        self._session: HTTPClient = session
        self._client: ClientManager = client

    async def gets(
        self,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
        *,
        folder_id: str,
    ) -> list[Folder]:
        """Get folders

        Endpoint: `/api/drive/folders`

        Parameters
        ----------
        limit: int
            The limit of folders to get, defaults to 10
        since_id: str | None
            The ID of the folder to get since, defaults to None
        until_id: str | None
            The ID of the folder to get until, defaults to None
        folder_id: str | None
            The ID of the folder to get, defaults to None

        Returns
        -------
        list[Folder]
            The found folders
        """
        data = {
            "limit": limit,
            "sinceId": since_id,
            "untilId": until_id,
            "folderId": folder_id,
        }
        raw_folders: list[IFolder] = await self._session.request(
            Route("POST", "/api/drive/folders"),
            auth=True,
            json=data,
        )

        return [Folder(raw_folder=raw_folder, client=self._client) for raw_folder in raw_folders]

    async def create(self, name: str | None = None, *, parent_id: str | None = None) -> Folder:
        """Create a new folder

        Endpoint: `/api/drive/folders/create`

        Parameters
        ----------
        name : str, optional
            The name of the folder, by default None
        parent_id : str, optional
            The parent ID of the folder, by default None

        Returns
        -------
        Folder
            The created folder
        """
        data = {"name": name, "parentId": parent_id}
        raw_created_folder: IFolder = await self._session.request(
            Route("POST", "/api/drive/folders/create"), auth=True, json=data
        )

        return Folder(raw_folder=raw_created_folder, client=self._client)

    async def delete(self, *, folder_id: str) -> bool:
        """Delete a folder

        Endpoint: `/api/drive/folders/delete`

        Parameters
        ----------
        folder_id : str, optional
            The ID of the folder, by default None

        Returns
        -------
        bool
            Whether the folder was deleted or not
        """
        res: bool = await self._session.request(
            Route("POST", "/api/drive/folders/delete"), auth=True, json={"folderId": folder_id}
        )

        return res

    async def update(
        self,
        name: str | None = MISSING,
        parent_id: str | None = MISSING,
        *,
        folder_id: str,
    ) -> Folder:
        """Update a folder

        Endpoint: `/api/drive/folders/update`

        Parameters
        ----------
        name : str, optional
            The name of the folder, by default MISSING
        parent_id : str, optional
            The parent ID of the folder, by default MISSING
        folder_id : str, optional
            The ID of the folder, by default None

        Returns
        -------
        Folder
            The updated folder
        """
        data = remove_dict_missing({"folderId": folder_id, "name": name, "parentId": parent_id})
        raw_updated_folder: IFolder = await self._session.request(
            Route("POST", "/api/drive/folders/update"), auth=True, json=data
        )

        return Folder(raw_folder=raw_updated_folder, client=self._client)


class ClientFolderActions(SharedFolderActions):
    def __init__(self, folder_id: str, *, session: HTTPClient, client: ClientManager):
        self.__folder_id: str = folder_id
        self._session: HTTPClient = session
        self._client: ClientManager = client

    @override
    async def gets(
        self,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
        *,
        folder_id: str | None = None,
    ) -> list[Folder]:
        """Get folders

        Endpoint: `/api/drive/folders`

        Parameters
        ----------
        limit: int
            The limit of folders to get, defaults to 10
        since_id: str | None
            The ID of the folder to get since, defaults to None
        until_id: str | None
            The ID of the folder to get until, defaults to None
        folder_id: str | None
            The ID of the folder to get, defaults to None

        Returns
        -------
        list[Folder]
            The found folders
        """
        folder_id = folder_id or self.__folder_id

        return await super().gets(
            limit=limit,
            since_id=since_id,
            until_id=until_id,
            folder_id=folder_id,
        )

    @override
    async def create(self, name: str | None = None, *, parent_id: str | None = None) -> Folder:
        """Create a new folder

        Endpoint: `/api/drive/folders/create`

        Parameters
        ----------
        name : str, optional
            The name of the folder, by default None
        parent_id : str, optional
            The parent ID of the folder, by default None

        Returns
        -------
        Folder
            The created folder
        """
        parent_id = parent_id or self.__folder_id

        return await super().create(name=name, parent_id=parent_id)

    @override
    async def delete(self, *, folder_id: str | None = None) -> bool:
        """Delete a folder

        Endpoint: `/api/drive/folders/delete`

        Parameters
        ----------
        folder_id : str, optional
            The ID of the folder, by default None

        Returns
        -------
        bool
            Whether the folder was deleted or not
        """
        folder_id = folder_id or self.__folder_id

        return await super().delete(folder_id=folder_id)

    @override
    async def update(
        self,
        name: str | None = MISSING,
        parent_id: str | None = MISSING,
        *,
        folder_id: str | None = None,
    ) -> Folder:
        """Update a folder

        Endpoint: `/api/drive/folders/update`

        Parameters
        ----------
        name : str, optional
            The name of the folder, by default MISSING
        parent_id : str, optional
            The parent ID of the folder, by default MISSING
        folder_id : str, optional
            The ID of the folder, by default None

        Returns
        -------
        Folder
            The updated folder
        """
        folder_id = folder_id or self.__folder_id

        return await super().update(name=name, parent_id=parent_id, folder_id=folder_id)


class FolderActions(SharedFolderActions):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        super().__init__(session=session, client=client)

    async def find(self, name: str, parent_id: str | None = None) -> list[Folder]:
        """Find folders

        Endpoint: `/api/drive/folders/find`

        Parameters
        ----------
        name : str
            The name of the folder
        parent_id : str, optional
            The parent ID of the folder, by default None

        Returns
        -------
        list[Folder]
            The found folders
        """

        data = {"name": name, "parentId": parent_id}
        raw_folders: list[IFolder] = await self._session.request(
            Route("POST", "/api/drive/folders/find"),
            auth=True,
            json=data,
        )

        return [Folder(raw_folder=raw_folder, client=self._client) for raw_folder in raw_folders]

    async def show(self, folder_id: str) -> Folder:
        """Show a folder

        Endpoint: `/api/drive/folders/show`

        Parameters
        ----------
        folder_id : str
            The ID of the folder

        Returns
        -------
        Folder
            The found folder
        """
        raw_folder: IFolder = await self._session.request(
            Route("POST", "/api/drive/folders/show"),
            auth=True,
            json={"folderId": folder_id},
        )

        return Folder(raw_folder=raw_folder, client=self._client)



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.manager import AbstractManager
from mipac.actions.users.mute import ClientMuteActions, MuteActions
from mipac.http import HTTPClient

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class ClientMuteManager(AbstractManager):
    def __init__(self, user_id: str, *, session: HTTPClient, client: ClientManager):
        self._action: ClientMuteActions = ClientMuteActions(
            user_id=user_id, session=session, client=client
        )

    @property
    def action(self) -> ClientMuteActions:
        return self._action


class MuteManager(AbstractManager):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        self._action: MuteActions = MuteActions(session=session, client=client)

    @property
    def action(self) -> MuteActions:
        return self._action



from __future__ import annotations

from typing import Any, Literal

from mipac.http import HTTPClient, Route


class Pagination[T]:
    def __init__(
        self,
        http_client: HTTPClient,
        route: Route,
        json: dict[str, Any],
        auth: bool = True,
        remove_none: bool = True,
        lower: bool = True,
        pagination_type: Literal["until", "count"] = "until",
        limit: int = 100,
    ) -> None:
        self.http_client: HTTPClient = http_client
        self.route: Route = route
        self.json: dict[str, Any] = json
        self.auth: bool = auth
        self.remove_none: bool = remove_none
        self.lower: bool = lower
        self.pagination_type: Literal["until", "count"] = pagination_type
        self.limit: int = limit
        self.count = 0
        self.next_id: str = ""
        self.latest_res_count: int | None = None

    async def next(self) -> list[T]:
        if self.pagination_type == "count":
            self.json["offset"] = self.json.get("limit", self.limit) * self.count
            self.count += 1
        res: list[T] = await self.http_client.request(
            self.route,
            auth=self.auth,
            remove_none=self.remove_none,
            lower=self.lower,
            json=self.json,
        )
        if self.pagination_type == "until":
            if len(res) > 0:
                self.next_id = res[-1]["id"]  # type: ignore
            self.json["untilId"] = self.next_id
        self.latest_res_count = len(res)
        return res

    @property
    def is_final(self) -> bool:
        if self.latest_res_count is None:
            return False

        match self.pagination_type:
            case "count":
                return self.latest_res_count == 0
            case "until":
                return self.latest_res_count == 0
            case _:
                raise ValueError("Invalid pagination type")



from typing import Literal, TypedDict

IAntennaReceiveSource = Literal["home", "all", "users", "list"]


class IAntenna(TypedDict):
    id: str
    created_at: str
    name: str
    keywords: list[str]
    exclude_keywords: list[str]
    src: IAntennaReceiveSource
    user_list_id: str | None
    users: list[str]
    case_sensitive: bool
    local_only: bool
    notify: bool
    with_replies: bool
    with_file: bool
    is_active: bool
    has_unread_note: bool



from __future__ import annotations

from datetime import datetime
from typing import TYPE_CHECKING, Literal

from mipac.models.lite.ad import PartialAd
from mipac.types.ads import IAd
from mipac.utils.format import str_to_datetime

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class Ad(PartialAd[IAd]):
    def __init__(self, ad_data: IAd, *, client: ClientManager) -> None:
        super().__init__(ad_data, client=client)

    @property
    def expires_at(self) -> datetime:
        return str_to_datetime(self._raw_ad["expires_at"])

    @property
    def starts_at(self) -> datetime:
        return str_to_datetime(self._raw_ad["starts_at"])

    @property
    def priority(self) -> Literal["high" "middle" "low"]:
        return self._raw_ad["priority"]

    @property
    def memo(self) -> str | None:
        return self._raw_ad["memo"]



from __future__ import annotations

from datetime import datetime
from typing import TYPE_CHECKING, Any

from mipac.models.lite.role import PartialRole
from mipac.models.user import MeDetailed, UserDetailedNotMe, packed_user
from mipac.types.roles import IRole, IRolePolicies, IRoleUser
from mipac.utils.format import str_to_datetime

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class RoleUser:
    def __init__(self, role_user: IRoleUser, *, client: ClientManager) -> None:
        self.__role_user = role_user
        self.__client = client

    @property
    def id(self) -> str:
        return self.__role_user["id"]

    @property
    def created_at(self) -> datetime:
        return str_to_datetime(self.__role_user["created_at"])

    @property
    def user(self) -> UserDetailedNotMe | MeDetailed:
        return packed_user(self.__role_user["user"], client=self.__client)

    @property
    def expires_at(self) -> datetime | None:
        return (
            str_to_datetime(self.__role_user["expires_at"])
            if self.__role_user["expires_at"]
            else None
        )

    def _get(self, key: str) -> Any | None:
        return self.__role_user.get(key)

    def __eq__(self, __value: object) -> bool:
        return isinstance(__value, RoleUser) and self.id == __value.id

    def __ne__(self, __value: object) -> bool:
        return not self.__eq__(__value)


class RolePolicies:
    def __init__(self, role_policies_data: IRolePolicies) -> None:
        self.__role_policies_data = role_policies_data

    @property
    def gtl_available(self) -> bool:
        return self.__role_policies_data["gtl_available"]

    @property
    def ltl_available(self) -> bool:
        return self.__role_policies_data["ltl_available"]

    @property
    def can_public_note(self) -> bool:
        return self.__role_policies_data["can_public_note"]

    @property
    def can_invite(self) -> bool:
        return self.__role_policies_data["can_invite"]

    @property
    def invite_limit(self) -> int:
        return self.__role_policies_data["invite_limit"]

    @property
    def invite_limit_cycle(self) -> int:
        return self.__role_policies_data["invite_limit_cycle"]

    @property
    def invite_expiration_time(self) -> int:
        return self.__role_policies_data["invite_expiration_time"]

    @property
    def can_manage_custom_emojis(self) -> bool:
        return self.__role_policies_data["can_manage_custom_emojis"]

    @property
    def can_manage_avatar_decorations(self) -> bool:
        return self.__role_policies_data["can_manage_avatar_decorations"]

    @property
    def can_search_notes(self) -> bool:
        return self.__role_policies_data["can_search_notes"]

    @property
    def can_use_translator(self) -> bool:
        return self.__role_policies_data["can_use_translator"]

    @property
    def can_hide_ads(self) -> bool:
        return self.__role_policies_data["can_hide_ads"]

    @property
    def drive_capacity_mb(self) -> int:
        return self.__role_policies_data["drive_capacity_mb"]

    @property
    def always_mark_nfsw(self) -> bool:
        return self.__role_policies_data["always_mark_nfsw"]

    @property
    def pin_limit(self) -> int:
        return self.__role_policies_data["pin_limit"]

    @property
    def antenna_limit(self) -> int:
        return self.__role_policies_data["antenna_limit"]

    @property
    def word_mute_limit(self) -> int:
        return self.__role_policies_data["word_mute_limit"]

    @property
    def webhook_limit(self) -> int:
        return self.__role_policies_data["webhook_limit"]

    @property
    def clip_limit(self) -> int:
        return self.__role_policies_data["clip_limit"]

    @property
    def note_each_clips_limit(self) -> int:
        return self.__role_policies_data["note_each_clips_limit"]

    @property
    def user_list_limit(self) -> int:
        return self.__role_policies_data["user_list_limit"]

    @property
    def user_each_user_lists_limit(self) -> int:
        return self.__role_policies_data["user_each_user_lists_limit"]

    @property
    def rate_limit_factor(self) -> int:
        return self.__role_policies_data["rate_limit_factor"]

    @property
    def avatar_decoration_limit(self) -> int:
        return self.__role_policies_data["avatar_decoration_limit"]

    def _get(self, key: str) -> Any | None:
        return self.__role_policies_data.get(key)


class Role(PartialRole[IRole]):
    def __init__(self, role_data: IRole, *, client: ClientManager) -> None:
        super().__init__(role_data, client=client)

    @property
    def created_at(self) -> str:
        return self._raw_role["created_at"]

    @property
    def updated_at(self) -> str:
        return self._raw_role["updated_at"]

    @property
    def target(self) -> str:
        return self._raw_role["target"]

    @property
    def cond_formula(self) -> dict:
        return self._raw_role["cond_formula"]

    @property
    def is_public(self) -> bool:
        return self._raw_role["is_public"]

    @property
    def is_explorable(self) -> bool:
        return self._raw_role["is_explorable"]

    @property
    def as_badge(self) -> bool:
        return self._raw_role["as_badge"]

    @property
    def can_edit_members_by_moderator(self) -> bool:
        return self._raw_role["can_edit_members_by_moderator"]

    @property
    def policies(self) -> RolePolicies:
        return RolePolicies(self._raw_role["policies"])

    @property
    def users_count(self) -> int:
        return self._raw_role["users_count"]

    def _get(self, key: str) -> Any | None:
        return self._raw_role.get(key)



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from mipac.types.roles import IPartialRole

if TYPE_CHECKING:
    from mipac.manager.admins.roles import AdminRolesModelManager
    from mipac.manager.client import ClientManager


class PartialRole[T: IPartialRole]:
    def __init__(self, role_data: T, *, client: ClientManager) -> None:
        self._raw_role: T = role_data
        self.__client = client

    @property
    def id(self) -> str:
        return self._raw_role["id"]

    @property
    def name(self) -> str:
        return self._raw_role["name"]

    @property
    def color(self) -> str | None:
        return self._raw_role["color"]

    @property
    def icon_url(self) -> str | None:
        return self._raw_role["icon_url"]

    @property
    def description(self) -> str:
        return self._raw_role["description"]

    @property
    def is_moderator(self) -> bool:
        return self._raw_role["is_moderator"]

    @property
    def is_administrator(self) -> bool:
        return self._raw_role["is_administrator"]

    @property
    def display_order(self) -> int:
        return self._raw_role["display_order"]

    @property
    def api(self) -> AdminRolesModelManager:
        return self.__client.admin.create_roles_model_manager(self.id)

    def _get(self, key: str) -> Any | None:
        return self._raw_role.get(key)

    def __eq__(self, __value: object) -> bool:
        return isinstance(__value, PartialRole) and self.id == __value.id

    def __ne__(self, __value: object) -> bool:
        return not self.__eq__(__value)



import asyncio
import uuid
from typing import Any, Literal, NotRequired, TypedDict
from urllib.parse import urlencode, urlparse, urlunparse

import aiohttp

from mipac.types.permission import Permissions
from mipac.types.user import IUserDetailed
from mipac.utils.format import remove_dict_empty, upper_to_lower


class IMiAuthPayload(TypedDict):
    ok: bool
    token: NotRequired[str]
    user: NotRequired[IUserDetailed]


class MiAuth:
    """Misskey v12以降のインスタンスで使用可能な認証方式です"""

    def __init__(self, host: str, protocol: Literal["http", "https"]) -> None:
        self.host: str = host
        self.protocol: Literal["http", "https"] = protocol
        self.session: str

        self.set_session()

    @property
    def url(self) -> str:
        return f"{self.protocol}://{self.host}"

    def set_session(self) -> None:
        self.session = uuid.uuid4().hex

    async def gen_session(
        self,
        name: str | None = None,
        icon: str | None = None,
        callback: str | None = None,
        permission: list[Permissions] | None = None,
    ):
        # sessionは使いまわししてはいけないのでこのタイミングで新しいsessionを生成する
        self.set_session()

        query = remove_dict_empty(
            {
                "name": name,
                "icon": icon,
                "callback": callback,
                "permission": ",".join(permission) if permission else None,
            }
        )

        url = urlparse(f"{self.url}/miauth/{self.session}")
        return urlunparse(
            (url.scheme, url.netloc, url.path, url.params, urlencode(query), url.fragment)
        )

    async def check_session(self) -> Any:
        async with aiohttp.ClientSession() as session:
            res = await session.post(
                f"{self.url}/api/miauth/{self.session}/check",
                json={},
                headers={"Content-Type": "application/json"},
            )

            if res.status != 200:
                raise Exception(f"MiAuth Check failed, Status code: {res.status}")

            data = await res.json()

            if data is None:
                raise Exception("MiAuth Check failed, Response is None")
            return upper_to_lower(data)

    async def wait_complete(self):
        while True:
            is_complete: IMiAuthPayload = await self.check_session()
            if is_complete["ok"] is True:
                break
            await asyncio.sleep(1)

        return is_complete



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.action import AbstractAction
from mipac.http import HTTPClient, Route

if TYPE_CHECKING:
    from mipac.client import ClientManager


class AdminAccountActions(AbstractAction):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        self.__session: HTTPClient = session
        self.__client: ClientManager = client

    async def create(self, username: str, password: str):  # TODO: 多分UserDetailed + tokenってキー
        """ユーザーを作成します

        Endpoint: `/api/admin/accounts/create`

        Parameters
        ----------
        username : str
            ユーザー名
        password : str
            パスワード
        """

        data = {"username": username, "password": password}
        res = await self.__session.request(
            Route("POST", "/api/admin/accounts/create"),
            json=data,
        )
        return res

    async def delete(self, *, user_id: str) -> bool:
        """対象のユーザーを削除します

        Endpoint: `/api/admin/accounts/delete`

        Parameters
        ----------
        user_id : str
            対象のユーザーID

        Returns
        -------
        bool
            成功ならTrue
        """
        res = await self._session.request(
            Route("POST", "/api/admin/accounts/delete"),
            json={"userId": user_id},
            auth=True,
            lower=True,
        )
        return bool(res)



from __future__ import annotations

from typing import Literal, NotRequired, TypedDict, TypeVar

from mipac.types.drive import IFile
from mipac.types.emoji import ICustomEmojiLite
from mipac.types.poll import IPoll
from mipac.types.reaction import IReactionAcceptance
from mipac.types.user import IPartialUser

T = TypeVar("T")

INoteVisibility = Literal["public", "home", "followers", "specified"]


class INoteState(TypedDict):
    is_favorited: bool
    is_muted_thread: bool


class INoteUpdated[T](TypedDict):
    type: Literal["noteUpdated"]
    body: T


class INoteUpdatedDeleteBody(TypedDict):
    deleted_at: str


class INoteUpdatedDelete(TypedDict):
    id: str
    type: Literal["deleted"]
    body: INoteUpdatedDeleteBody


class INoteUpdatedReactionBody(TypedDict):
    reaction: str
    emoji: ICustomEmojiLite
    user_id: str


class INoteUpdatedReaction(TypedDict):
    id: str
    type: Literal["reacted", "unreacted"]
    body: INoteUpdatedReactionBody


class INoteChannel(TypedDict):
    """ノート内にあるチャンネルの情報

    ログインしていてもis_following等は存在しない
    """

    id: str
    name: str
    color: str
    is_sensitive: bool
    allow_renote_to_external: bool
    user_id: str | None


class INote(TypedDict):
    """Misskey Raw Model: Note"""

    id: str
    created_at: str
    deleted_at: NotRequired[str]
    text: str | None
    cw: str | None
    user_id: str
    user: IPartialUser
    reply_id: str | None
    renote_id: str | None
    reply: NotRequired["INote"]
    renote: NotRequired["INote"]
    is_hidden: NotRequired[bool]
    visibility: INoteVisibility
    mentions: NotRequired[list[str]]
    visible_user_ids: NotRequired[list[str]]
    file_ids: list[str]
    files: list[IFile]
    tags: NotRequired[list[str]]
    poll: NotRequired[IPoll]
    emojis: dict[str, str]
    channel_id: NotRequired[str | None]
    channel: NotRequired[INoteChannel | None]
    local_only: bool
    reaction_acceptance: IReactionAcceptance
    reactions: dict[str, int]  # リアクションの種類と数
    renote_count: int
    replies_count: int
    uri: NotRequired[str]
    url: NotRequired[str]
    reaction_and_user_pair_cache: NotRequired[
        dict[str, list[IPartialUser]]
    ]  # リアクションとユーザーのペアのキャッシュ
    clipped_count: NotRequired[int]  # Misskeyの内部的にたまに存在しないだけで普通は存在しそう...?
    my_reaction: NotRequired[str | None]  # ログイン時のみ存在


class ICreatedNote(TypedDict):
    """
    created note
    """

    created_note: INote


class INoteReaction(TypedDict):
    id: str
    created_at: str
    user: IPartialUser
    type: str


class INoteTranslateResult(TypedDict):
    sourceLang: str
    text: str



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.manager import AbstractManager
from mipac.actions.drive.folders import (
    ClientFileActionsInFolder,
    ClientFolderActions,
    FolderActions,
)
from mipac.http import HTTPClient

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class ClientFolderManager(AbstractManager):
    def __init__(self, folder_id: str, *, session: HTTPClient, client: ClientManager):
        self.__folder_id: str = folder_id
        self.__session: HTTPClient = session
        self.__client: ClientManager = client
        self.__action: ClientFolderActions = ClientFolderActions(
            folder_id=folder_id, session=session, client=client
        )
        self.files = ClientFileActionsInFolder(folder_id=folder_id, session=session, client=client)

    @property
    def action(self) -> ClientFolderActions:
        return self.__action


class FolderManager(AbstractManager):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        self.__session: HTTPClient = session
        self.__client: ClientManager = client
        self.__action: FolderActions = FolderActions(session=session, client=client)

    @property
    def action(self) -> FolderActions:
        return self.__action



from typing import Literal, NotRequired, TypedDict

AnnoucementIcon = Literal["info", "wanirng", "error", "success"]
AnnoucementDisplay = Literal["dialog", "normal", "banner"]


class IAnnouncement(TypedDict):
    id: str
    created_at: str
    updated_at: str | None
    text: str
    title: str
    image_url: str | None
    icon: AnnoucementIcon
    display: AnnoucementDisplay
    need_confirmation_to_read: bool
    silence: bool
    for_you: bool
    is_read: NotRequired[bool]


class IAnnouncementDetailed(TypedDict):
    """管理者から見たアナウンス"""

    id: str
    created_at: str
    updated_at: str | None
    text: str
    title: str
    image_url: str | None
    icon: AnnoucementIcon
    display: AnnoucementDisplay
    need_confirmation_to_read: bool
    silence: bool

    is_active: bool
    for_existing_users: bool
    user_id: str | None
    reads: int



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.manager import AbstractManager
from mipac.actions.channel import ChannelActions, ClientChannelActions
from mipac.http import HTTPClient

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class ClientChannelManager(AbstractManager):
    def __init__(self, channel_id: str, *, session: HTTPClient, client: ClientManager):
        self._channel_id: str = channel_id
        self._session: HTTPClient = session
        self._client: ClientManager = client
        self._action: ClientChannelActions = ClientChannelActions(
            channel_id=channel_id, session=session, client=client
        )

    @property
    def action(self) -> ClientChannelActions:
        return self._action


class ChannelManager(AbstractManager):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        self.__session: HTTPClient = session
        self.__client: ClientManager = client
        self.__action: ChannelActions = ChannelActions(
            session=self.__session, client=self.__client
        )

    @property
    def action(self) -> ChannelActions:
        return self.__action



from __future__ import annotations

from datetime import datetime
from typing import TYPE_CHECKING, Any

from mipac.models.drive import File
from mipac.models.lite.user import PartialUser
from mipac.models.poll import Poll
from mipac.types.note import (
    INote,
    INoteChannel,
    INoteReaction,
    INoteState,
    INoteTranslateResult,
    INoteUpdated,
    INoteUpdatedDelete,
    INoteVisibility,
)
from mipac.types.reaction import IReactionAcceptance
from mipac.utils.format import str_to_datetime
from mipac.utils.util import deprecated

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager
    from mipac.manager.note import ClientNoteManager


__all__ = (
    "NoteState",
    "Note",
    "NoteReaction",
    "NoteDeleted",
    "NoteTranslateResult",
)


class NoteState:
    def __init__(self, data: INoteState) -> None:
        self.__data: INoteState = data

    @property
    def is_favorite(self) -> bool:
        return self.__data["is_favorited"]

    @property
    def is_muted_thread(self) -> bool:
        return self.__data["is_muted_thread"]

    def _get(self, key: str) -> Any | None:
        return self.__data.get(key)


class NoteDeleted:
    def __init__(self, data: INoteUpdated[INoteUpdatedDelete]) -> None:
        self.__data = data

    @property
    def note_id(self) -> str:
        return self.__data["body"]["id"]

    @property
    def deleted_at(self) -> datetime:
        return str_to_datetime(self.__data["body"]["body"]["deleted_at"])

    def _get(self, key: str) -> Any | None:
        return self.__data.get(key)

    def __eq__(self, __value: object) -> bool:
        return isinstance(__value, NoteDeleted) and self.note_id == __value.note_id

    def __ne__(self, __value: object) -> bool:
        return not self.__eq__(__value)


class NoteReaction:
    __slots__ = ("__reaction", "__client")

    def __init__(self, raw_reaction: INoteReaction, *, client: ClientManager):
        self.__reaction: INoteReaction = raw_reaction
        self.__client: ClientManager = client

    @property
    def id(self) -> str | None:
        """note reactionId

        Returns
        -------
        str | None
            note reactionId
        """
        return self.__reaction["id"]

    @property
    def created_at(self) -> datetime | None:
        """note createdAt

        Returns
        -------
        datetime | None
            note createdAt
        """
        return (
            str_to_datetime(self.__reaction["created_at"])
            if "created_at" in self.__reaction
            else None
        )

    @property
    def user(self) -> PartialUser:
        """note user

        Returns
        -------
        PartialUser
            note user
        """
        return PartialUser(self.__reaction["user"], client=self.__client)

    @property
    def type(self) -> str | None:
        """reaction type

        Returns
        -------
        str | None
            reaction type
        """
        return self.__reaction["type"]

    def _get(self, key: str) -> Any | None:
        return self.__reaction.get(key)

    def __eq__(self, __value: object) -> bool:
        return isinstance(__value, NoteReaction) and self.id == __value.id

    def __ne__(self, __value: object) -> bool:
        return not self.__eq__(__value)


class NoteChannel:
    def __init__(self, raw_note_channel: INoteChannel, *, client: ClientManager) -> None:
        self.__raw_note_channel: INoteChannel = raw_note_channel
        self.__client: ClientManager = client

    @property
    def id(self) -> str:
        """note channelId

        Returns
        -------
        str
            note channelId
        """
        return self.__raw_note_channel["id"]

    @property
    def name(self) -> str:
        """note channelName

        Returns
        -------
        str
            note channelName
        """
        return self.__raw_note_channel["name"]

    @property
    def color(self) -> str:
        """note channelColor

        Returns
        -------
        str
            note channelColor
        """
        return self.__raw_note_channel["color"]

    @property
    def is_sensitive(self) -> bool:
        """note channelIsSensitive

        Returns
        -------
        bool
            note channelIsSensitive
        """
        return self.__raw_note_channel["is_sensitive"]

    @property
    def allow_renote_to_external(self) -> bool:
        """note channelAllowRenoteToExternal

        Returns
        -------
        bool
            note channelAllowRenoteToExternal
        """
        return self.__raw_note_channel["allow_renote_to_external"]

    @property
    def user_id(self) -> str | None:
        """note channelUserId

        Returns
        -------
        str | None
            note channelUserId
        """
        return self.__raw_note_channel["user_id"]

    def _get(self, key: str) -> Any | None:
        return self.__raw_note_channel.get(key)


class Note:
    """Noteモデル

    Parameters
    ----------
    note: INote
        The raw data of the note

    client: ClientManager
    """

    def __init__(self, raw_note: INote, client: ClientManager):
        self.__raw_note: INote = raw_note
        self.__client: ClientManager = client

    @property
    def id(self) -> str:
        """
        note id

        Returns
        -------
        str
            note id
        """
        return self.__raw_note["id"]

    @property
    def created_at(self) -> datetime:
        """note createdAt

        Returns
        -------
        datetime
            note createdAt
        """
        return str_to_datetime(self.__raw_note["created_at"])

    @property
    def deleted_at(self) -> datetime | None:
        """note deletedAt

        Returns
        -------
        datetime | None
            note deletedAt
        """
        return (
            str_to_datetime(self.__raw_note["deleted_at"])
            if "deleted_at" in self.__raw_note
            else None
        )

    @property
    def text(self) -> str | None:
        """note text

        Returns
        -------
        str | None
            note text
        """
        return self.__raw_note.get("text")

    @property
    @deprecated
    def content(self) -> str | None:
        """note content

        .. deprecated:: 0.6.0
            Use :meth:`mipac.models.note.Note.text` instead.

        Returns
        -------
        str | None
            note content
        """
        return self.text

    @property
    def cw(self) -> str | None:
        """note cw

        Returns
        -------
        str | None
            note cw
        """
        return self.__raw_note.get("cw")

    @property
    def user_id(self) -> str:
        """note userId

        Returns
        -------
        str
            note userId
        """
        return self.__raw_note["user_id"]

    @property
    @deprecated
    def author(self) -> PartialUser:
        """note author

        .. deprecated:: 0.6.0
            Use :meth:`mipac.models.note.Note.user` instead.


        Returns
        -------
        PartialUser
            note author
        """
        return PartialUser(self.__raw_note["user"], client=self.__client)

    @property
    def user(self) -> PartialUser:
        """note author

        Returns
        -------
        PartialUser
            note user
        """
        return PartialUser(self.__raw_note["user"], client=self.__client)

    @property
    def reply_id(self) -> str | None:
        """note replyId

        Returns
        -------
        str | None
            note replyId
        """
        return self.__raw_note["reply_id"]

    @property
    def renote_id(self) -> str | None:
        """note renoteId

        Returns
        -------
        str | None
            note renoteId
        """
        return self.__raw_note["renote_id"]

    @property
    def reply(self) -> Note | None:
        """note reply

        Returns
        -------
        Note | None
            note reply
        """
        return (
            Note(self.__raw_note["reply"], client=self.__client)
            if "reply" in self.__raw_note
            else None
        )

    @property
    def renote(self) -> Note | None:
        """note renote

        Returns
        -------
        Note | None
            note renote
        """
        return (
            Note(self.__raw_note["renote"], client=self.__client)
            if "renote" in self.__raw_note
            else None
        )

    @property
    def is_hidden(self) -> bool | None:
        """note isHidden

        Returns
        -------
        bool | None
            note isHidden
        """
        return self.__raw_note.get("is_hidden")

    @property
    def visibility(self) -> INoteVisibility:
        """note visibility

        Returns
        -------
        INoteVisibility
            note visibility
        """
        return self.__raw_note["visibility"]

    @property
    def mentions(self) -> list[str]:
        """note mentions

        Returns
        -------
        list[str]
            note mentions
        """
        return self.__raw_note.get("mentions", [])

    @property
    def visible_user_ids(self) -> list[str]:
        """note visibleUserIds

        Returns
        -------
        list[str] | None
            note visibleUserIds
        """
        return self.__raw_note.get("visible_user_ids", [])

    @property
    def file_ids(self) -> list[str]:
        """note fileIds

        Returns
        -------
        list[str]
            note fileIds
        """
        return self.__raw_note["file_ids"]

    @property
    def files(self) -> list[File]:
        """note files

        Returns
        -------
        list[IFile]
            note files
        """
        return [File(raw_file, client=self.__client) for raw_file in self.__raw_note["files"]]

    @property
    def tags(self) -> list[str]:
        """note tags

        Returns
        -------
        list[str] | None
            note tags
        """
        return self.__raw_note.get("tags", [])

    @property
    def poll(self) -> Poll | None:
        """note poll

        Returns
        -------
        IPoll | None
            note poll
        """
        return (
            Poll(self.__raw_note["poll"], client=self.__client)
            if "poll" in self.__raw_note
            else None
        )

    @property
    def emojis(self) -> dict[str, str]:
        """note emojis

        Returns
        -------
        dict[str, str]
            note emojis
        """
        return self.__raw_note["emojis"]

    @property
    def channel_id(self) -> str | None:
        """note channelId

        Returns
        -------
        str | None
            note channelId
        """
        return self.__raw_note.get("channel_id")

    @property
    def channel(self) -> NoteChannel | None:
        """note channel

        Returns
        -------
        NoteChannel | None
            note channel
        """
        return (
            NoteChannel(self.__raw_note["channel"], client=self.__client)
            if "channel" in self.__raw_note and self.__raw_note["channel"]
            else None
        )

    @property
    def local_only(self) -> bool:
        """note localOnly

        Returns
        -------
        bool
            note localOnly
        """
        return self.__raw_note["local_only"]

    @property
    def reaction_acceptance(self) -> IReactionAcceptance:
        """note reactionAcceptance

        Returns
        -------
        IReactionAcceptance
            note reactionAcceptance
        """
        return self.__raw_note["reaction_acceptance"]

    @property
    def reactions(self) -> dict[str, int]:
        """note reactions

        Returns
        -------
        dict[str, int]
            note reactions
        """
        return self.__raw_note["reactions"]

    @property
    def renote_count(self) -> int:
        """note renoteCount

        Returns
        -------
        int
            note renoteCount
        """
        return self.__raw_note["renote_count"]

    @property
    def replies_count(self) -> int:
        """note repliesCount

        Returns
        -------
        int
            note repliesCount
        """
        return self.__raw_note["replies_count"]

    @property
    def uri(self) -> str | None:
        """note uri

        Returns
        -------
        str
            note uri
        """
        return self.__raw_note.get("uri")

    @property
    def url(self) -> str | None:
        """note url

        Returns
        -------
        str
            note url
        """
        return self.__raw_note.get("url")

    @property
    def reaction_and_user_pair_cache(self) -> dict[str, list[PartialUser]]:
        """note reactionAndUserPairCache

        Returns
        -------
        dict[str, list[PartialUser]]
            note reactionAndUserPairCache
        """
        if "reaction_and_user_pair_cache" not in self.__raw_note:
            return {}
        reaction_and_user_pair_cache = {}

        for k, v in self.__raw_note["reaction_and_user_pair_cache"].items():
            reaction_and_user_pair_cache[k] = [
                PartialUser(user, client=self.__client) for user in v
            ]
        return reaction_and_user_pair_cache

    @property
    def clipped_count(self) -> int | None:
        """note clippedCount

        Returns
        -------
        int | None
            note clippedCount
        """
        return self.__raw_note.get("clipped_count")

    @property
    def my_reaction(self) -> str | None:
        """note myReaction

        Returns
        -------
        str | None
            note myReaction
        """
        return self.__raw_note.get("my_reaction")

    @property
    def api(self) -> ClientNoteManager:
        """note api

        Returns
        -------
        ClientNoteManager
            note api
        """
        return self.__client._create_client_note_manager(note_id=self.id)

    def _get(self, key: str) -> Any | None:
        return self.__raw_note.get(key)

    def __eq__(self, __value: object) -> bool:
        return isinstance(__value, Note) and self.id == __value.id

    def __ne__(self, __value: object) -> bool:
        return not self.__eq__(__value)


class NoteTranslateResult:
    """
    NoteTranslateResult

    Parameters
    ----------
    translate_result: INoteTranslateResult
        The raw data of the note translate result
    """

    def __init__(self, translate_result: INoteTranslateResult):
        self.__translate_result = translate_result

    @property
    def source_language(self) -> str:
        return self.__translate_result["sourceLang"]

    @property
    def text(self) -> str:
        return self.__translate_result["text"]

    def _get(self, key: str) -> Any | None:
        return self.__translate_result.get(key)



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.manager import AbstractManager
from mipac.actions.admins.admin import AdminActions
from mipac.http import HTTPClient
from mipac.manager.admins.accounts import AdminAccountManager
from mipac.manager.admins.ad import AdminAdManager, ClientAdminAdManager
from mipac.manager.admins.announcement import (
    AdminAnnouncementManager,
    ClientAdminAnnouncementManager,
)
from mipac.manager.admins.drive import AdminDriveManager
from mipac.manager.admins.emoji import AdminEmojiManager
from mipac.manager.admins.invite import AdminInviteManager
from mipac.manager.admins.roles import AdminRolesManager, AdminRolesModelManager
from mipac.manager.admins.user import AdminUserManager

if TYPE_CHECKING:
    from mipac.client import ClientManager


class AdminManager(AbstractManager):
    def __init__(self, session: HTTPClient, client: ClientManager):
        self.__session: HTTPClient = session
        self.__client: ClientManager = client
        self.emoji: AdminEmojiManager = AdminEmojiManager(session=session, client=client)
        self.user: AdminUserManager = AdminUserManager(session=session, client=client)
        self.ad: AdminAdManager = AdminAdManager(session=session, client=client)
        self.announcement: AdminAnnouncementManager = AdminAnnouncementManager(
            session=session, client=client
        )
        self.role: AdminRolesManager = AdminRolesManager(session=session, client=client)
        self.invite: AdminInviteManager = AdminInviteManager(session=session, client=client)
        self.drive: AdminDriveManager = AdminDriveManager(session=session, client=client)
        self.account: AdminAccountManager = AdminAccountManager(session=session, client=client)

    @property
    def action(self) -> AdminActions:
        return AdminActions(session=self.__session, client=self.__client)

    def create_roles_model_manager(self, role_id: str | None = None) -> AdminRolesModelManager:
        return AdminRolesModelManager(
            role_id=role_id, session=self.__session, client=self.__client
        )

    def _create_client_ad_manager(self, ad_id: str) -> ClientAdminAdManager:
        return ClientAdminAdManager(ad_id=ad_id, session=self.__session, client=self.__client)

    def _create_client_announcement_manager(
        self, announce_id: str
    ) -> ClientAdminAnnouncementManager:
        return ClientAdminAnnouncementManager(
            announce_id=announce_id,
            session=self.__session,
            client=self.__client,
        )



from __future__ import annotations

from typing import TYPE_CHECKING, override

from mipac.abstract.action import AbstractAction
from mipac.http import HTTPClient, Route
from mipac.models.mute import Muting
from mipac.types.mute import IMuting
from mipac.utils.pagination import Pagination

if TYPE_CHECKING:
    from mipac.client import ClientManager


class SharedMuteActions(AbstractAction):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        self._session: HTTPClient = session
        self._client: ClientManager = client

    async def create(self, expires_at: int | None = None, *, user_id: str) -> bool:
        """指定したユーザーをミュートします

        Parameters
        ----------
        user_id : str | None
            対象のユーザーID, default=None
        expires_at : int | None
            ミュートする期間(秒)、無期限でミュートする場合はNoneを指定します

        Returns
        -------
        bool
            ミュートに成功したかどうか
        """
        body = {"userId": user_id, "expiresAt": expires_at}

        res: bool = await self._session.request(route=Route("POST", "/api/mute/create"), json=body)

        return res

    async def delete(self, *, user_id: str) -> bool:
        """指定したユーザーのミュートを解除します

        Parameters
        ----------
        user_id : str | None
            対象のユーザーID, default=None

        Returns
        -------
        bool
            ミュート解除に成功したかどうか
        """
        res: bool = await self._session.request(
            route=Route("POST", "/api/mute/delete"), json={"userId": user_id}
        )

        return res


class ClientMuteActions(SharedMuteActions):
    def __init__(self, user_id: str, *, session: HTTPClient, client: ClientManager):
        super().__init__(session=session, client=client)
        self.__user_id: str = user_id

    @override
    async def create(self, expires_at: int | None = None, *, user_id: str | None = None) -> bool:
        """指定したユーザーをミュートします

        Parameters
        ----------
        user_id : str
            対象のユーザーID
        expires_at : int | None
            ミュートする期間(秒)、無期限でミュートする場合はNoneを指定します

        Returns
        -------
        bool
            ミュートに成功したかどうか
        """
        user_id = user_id or self.__user_id

        return await super().create(expires_at=expires_at, user_id=user_id or self.__user_id)

    @override
    async def delete(self, *, user_id: str | None = None) -> bool:
        """指定したユーザーのミュートを解除します

        Parameters
        ----------
        user_id : str
            対象のユーザーID

        Returns
        -------
        bool
            ミュート解除に成功したかどうか
        """
        user_id = user_id or self.__user_id

        return await super().delete(user_id=user_id or self.__user_id)


class MuteActions(SharedMuteActions):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        super().__init__(session=session, client=client)

    async def get_list(
        self, limit: int = 30, since_id: str | None = None, until_id: str | None = None
    ) -> list[Muting]:
        """ミュートしているユーザーの一覧を取得します

        Returns
        -------
        list[Muting]
            ミュートしているユーザーの一覧
        """

        body = {"limit": limit, "sinceId": since_id, "untilId": until_id}

        mutes: list[IMuting] = await self._session.request(
            Route("GET", "/api/mute/list"), json=body, auth=True
        )

        return [Muting(raw_mute_user=mute, client=self._client) for mute in mutes]

    async def get_all_list(
        self, limit: int = 30, since_id: str | None = None, until_id: str | None = None
    ):
        """ミュートしている全てのユーザーを取得します

        Parameters
        ----------
        limit : int
            取得するユーザー数, by default 30
        since_id : str | None
            指定したIDより後のミュート中のユーザーを取得します,  default=None
        until_id : str | None
            指定したIDより前のミュート中のユーザーを取得します, default=None

        Returns
        -------
        AsyncGenerator[Muting]
            ミュートしているユーザー
        """
        body = {"limit": limit, "sinceId": since_id, "untilId": until_id}

        pagination = Pagination[IMuting](
            self._session, Route("GET", "/api/mute/list"), json=body, auth=True
        )

        while pagination.is_final is False:
            for raw_muted_user in await pagination.next():
                yield Muting(raw_mute_user=raw_muted_user, client=self._client)



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.manager import AbstractManager
from mipac.actions.note import ClientNoteActions, NoteActions
from mipac.http import HTTPClient
from mipac.manager.favorite import ClientFavoriteManager, FavoriteManager
from mipac.manager.poll import ClientPollManager, PollManager
from mipac.manager.reaction import ClientReactionManager, ReactionManager

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class ClientNoteManager(AbstractManager):
    def __init__(self, note_id: str, *, session: HTTPClient, client: ClientManager):
        self.__note_id = note_id
        self.__session: HTTPClient = session
        self.__client: ClientManager = client
        self.reaction: ClientReactionManager = ClientReactionManager(
            note_id=note_id, session=session, client=client
        )
        self.favorite = ClientFavoriteManager(note_id=note_id, session=session, client=client)
        self.poll: ClientPollManager = ClientPollManager(
            note_id=note_id, session=session, client=client
        )
        self.__action: ClientNoteActions = ClientNoteActions(
            note_id=self.__note_id,
            session=self.__session,
            client=self.__client,
        )

    @property
    def action(self) -> ClientNoteActions:
        return self.__action


class NoteManager(AbstractManager):
    """User behavior for notes"""

    def __init__(self, *, session: HTTPClient, client: ClientManager):
        self.__session: HTTPClient = session
        self.__client: ClientManager = client
        self.reaction: ReactionManager = ReactionManager(session=session, client=client)
        self.favorite = FavoriteManager(session=session, client=client)
        self.poll: PollManager = PollManager(session=session, client=client)
        self.__action: NoteActions = NoteActions(
            session=self.__session,
            client=self.__client,
        )  # property側で生成するとcacheが効かなくなる

    @property
    def action(self) -> NoteActions:
        return self.__action



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.action import AbstractAction
from mipac.http import HTTPClient, Route
from mipac.models.drive import DriveStatus, File
from mipac.types.drive import IDriveStatus, IFile

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class DriveActions(AbstractAction):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        self.__session: HTTPClient = session
        self.__client: ClientManager = client

    async def get_status(self) -> DriveStatus:
        """Get the status of the drive

        Endpoint: `/api/drive`

        Returns
        -------
        DriveStatus
            The status of the drive
        """

        res: IDriveStatus = await self.__session.request(Route("POST", "/api/drive"), auth=True)
        return DriveStatus(raw_drive_status=res, client=self.__client)

    async def stream(
        self,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
        type: str | None = None,
    ) -> list[File]:
        """Stream files from the drive

        Endpoint: `/api/drive/stream`

        Parameters
        ----------
        limit: int
            The number of files to get
        since_id: str
            The id of the file to start from
        until_id: str
            The id of the file to end at
        type: str
            The type of file to get

        Returns
        -------
        list[File]
            A list of files
        """

        params = {
            "limit": limit,
            "sinceId": since_id,
            "untilId": until_id,
            "type": type,
        }
        raw_files: list[IFile] = await self.__session.request(
            Route("POST", "/api/drive/stream"), auth=True, json=params
        )

        return [File(raw_file=raw_file, client=self.__client) for raw_file in raw_files]



from typing import NotRequired, TypedDict

__all__ = ("IPoll", "IPollChoice", "ICreatePoll")


class IPollChoice(TypedDict):
    is_voted: bool
    text: str
    votes: int


class IPoll(TypedDict):
    """
    Questionnaire object
    """

    expires_at: str
    multiple: bool
    choices: list[IPollChoice]


class ICreatePoll(TypedDict):
    choices: list[str]
    multiple: NotRequired[bool]
    expires_at: NotRequired[int]
    expired_after: NotRequired[int]



from typing import TypedDict

from mipac.types.user import IUserDetailedNotMeSchema


class IBlocking(TypedDict):
    id: str
    created_at: str
    blockee_id: str
    blockee: IUserDetailedNotMeSchema



from __future__ import annotations

from typing import TYPE_CHECKING, override

from mipac.abstract.action import AbstractAction
from mipac.http import HTTPClient, Route
from mipac.models.note import Note
from mipac.types.note import INote
from mipac.utils.pagination import Pagination

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class SharedPollActions(AbstractAction):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        self._session: HTTPClient = session
        self._client: ClientManager = client

    async def vote(
        self,
        choice: int,
        *,
        note_id: str,
    ) -> bool:
        data = {"noteId": note_id, "choice": choice}
        res: bool = await self._session.request(
            Route("POST", "/api/notes/polls/vote"), auth=True, json=data
        )
        return res


class ClientPollActions(SharedPollActions):
    def __init__(self, note_id: str, *, session: HTTPClient, client: ClientManager):
        super().__init__(session=session, client=client)
        self._note_id: str = note_id

    @override
    async def vote(self, choice: int, *, note_id: str | None = None) -> bool:
        note_id = note_id or self._note_id

        return await super().vote(choice=choice, note_id=note_id)


class PollActions(SharedPollActions):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        super().__init__(session=session, client=client)

    async def recommendation(self, limit: int = 100, offset: int = 0):
        if limit > 100:
            raise ValueError("limit must be less than 100")

        data = {"limit": limit, "offset": offset}

        res: list[INote] = await self._session.request(
            Route("POST", "/api/notes/polls/recommendation"), json=data, auth=True
        )
        return [Note(note, client=self._client) for note in res]

    async def get_all_recommendation(self, offset: int = 0):
        limit = 100

        data = {"limit": limit, "offset": offset}

        pagination = Pagination[INote](
            self._session,
            Route("POST", "/api/notes/polls/recommendation"),
            json=data,
            pagination_type="count",
            auth=True,
        )

        while True:
            res_notes = await pagination.next()
            for note in res_notes:
                yield Note(note, client=self._client)
            if pagination.is_final:
                break



from __future__ import annotations

from typing import TYPE_CHECKING, override

from mipac.abstract.action import AbstractAction
from mipac.http import HTTPClient, Route
from mipac.models.note import Note
from mipac.models.user import UserList, UserListMembership
from mipac.types.user import IUserList, IUserListMembership
from mipac.utils.format import remove_dict_missing
from mipac.utils.pagination import Pagination
from mipac.utils.util import MISSING

if TYPE_CHECKING:
    from mipac.client import ClientManager


class SharedPartialUserListActions(AbstractAction):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        self._session: HTTPClient = session
        self._client: ClientManager = client

    async def get_list(self, *, user_id: str) -> list[UserList]:
        """Get the user lists of a user

        Endpoint `/api/users/lists/list`

        Parameters
        ----------
        user_id : str
            The id of the user to get the lists of

        Returns
        -------
        list[UserList]
            The user lists the user has
        """
        raw_user_lists: list[IUserList] = await self._session.request(
            Route("POST", "/api/users/lists/list"), json={"userId": user_id}, auth=True
        )

        return [UserList(raw_user_list, client=self._client) for raw_user_list in raw_user_lists]

    async def pull(self, *, list_id: str, user_id: str) -> bool:
        """Pull a user from a user list

        Endpoint `/api/users/lists/pull`

        Parameters
        ----------
        list_id : str
            The id of the user list to pull from
        user_id : str, optional
            The id of the user to pull, by default None

        Returns
        -------
        bool
            True if the user was pulled, False otherwise
        """
        res: bool = await self._session.request(
            Route("POST", "/api/users/lists/pull"),
            json={"listId": list_id, "userId": user_id},
            auth=True,
        )
        return res

    async def push(self, *, list_id: str, user_id: str) -> bool:
        """Push a user to a user list

        Endpoint `/api/users/lists/push`

        Parameters
        ----------
        list_id : str
            The id of the user list to push to
        user_id : str, optional
            The id of the user to push, by default None

        Returns
        -------
        bool
            True if the user was pushed, False otherwise
        """
        res: bool = await self._session.request(
            Route("POST", "/api/users/lists/push"),
            json={"listId": list_id, "userId": user_id},
            auth=True,
        )
        return res

    async def update_membership(self, with_replies: bool = MISSING, *, list_id: str, user_id: str):
        data = remove_dict_missing(
            {"listId": list_id, "userId": user_id, "withReplies": with_replies}
        )

        res: bool = await self._session.request(
            Route("POST", "/api/users/lists/update-membership"),
            json=data,
            auth=True,
        )
        return res


class ClientPartialUserListActions(SharedPartialUserListActions):
    """ユーザー向けのリストのアクションを提供します。"""

    def __init__(self, user_id: str, *, session: HTTPClient, client: ClientManager):
        super().__init__(session=session, client=client)
        self.__user_id: str = user_id

    @override
    async def get_list(self, *, user_id: str | None = None) -> list[UserList]:
        """Get the user lists of a user

        Endpoint `/api/users/lists/list`

        Parameters
        ----------
        user_id : str
            The id of the user to get the lists of

        Returns
        -------
        list[UserList]
            The user lists the user has
        """
        user_id = user_id or self.__user_id
        return await super().get_list(user_id=user_id)

    @override
    async def pull(self, *, list_id: str, user_id: str | None = None) -> bool:
        """Pull a user from a user list

        Endpoint `/api/users/lists/pull`

        Parameters
        ----------
        list_id : str
            The id of the user list to pull from
        user_id : str, optional
            The id of the user to pull, by default None

        Returns
        -------
        bool
            True if the user was pulled, False otherwise
        """
        user_id = user_id or self.__user_id

        return await super().pull(list_id=list_id, user_id=user_id)

    @override
    async def push(self, *, list_id: str, user_id: str | None = None) -> bool:
        """Push a user to a user list

        Endpoint `/api/users/lists/push`

        Parameters
        ----------
        list_id : str
            The id of the user list to push to
        user_id : str, optional
            The id of the user to push, by default None

        Returns
        -------
        bool
            True if the user was pushed, False otherwise
        """
        user_id = user_id or self.__user_id

        return await super().push(list_id=list_id, user_id=user_id)

    @override
    async def update_membership(
        self, with_replies: bool = MISSING, *, list_id: str, user_id: str | None = None
    ):
        user_id = user_id or self.__user_id

        return await super().update_membership(
            list_id=list_id, with_replies=with_replies, user_id=user_id
        )


class SharedUserListActions(SharedPartialUserListActions):
    def __init__(
        self,
        *,
        session: HTTPClient,
        client: ClientManager,
    ):
        super().__init__(session=session, client=client)

    async def delete(self, *, list_id: str) -> bool:
        """Delete a user list

        Endpoint `/api/users/lists/delete`

        Parameters
        ----------
        list_id : str, optional
            The id of the user list to delete, by default None

        Returns
        -------
        bool
            True if the user list was deleted, False otherwise
        """
        res: bool = await self._session.request(
            Route("POST", "/api/users/lists/delete"), json={"listId": list_id}, auth=True
        )
        return res

    async def show(self, for_public: bool = False, *, list_id: str) -> UserList:
        """Show a user list

        Endpoint `/api/users/lists/show`

        Parameters
        ----------
        for_public : bool, optional
            Whether to show the user list for the public, by default False
        list_id : str, optional
            The id of the user list to show, by default None

        Returns
        -------
        UserList
            The user list
        """
        raw_user_list: IUserList = await self._session.request(
            Route("POST", "/api/users/lists/show"),
            json={"listId": list_id, "forPublic": for_public},
            auth=True,
        )
        return UserList(raw_user_list=raw_user_list, client=self._client)

    async def favorite(self, *, list_id: str) -> bool:
        """Favorite a user list

        Endpoint `/api/users/lists/favorite`

        Parameters
        ----------
        list_id : str, optional
            The id of the user list to favorite, by default None

        Returns
        -------
        bool
            True if the user list was favorited, False otherwise
        """
        res: bool = await self._session.request(
            Route("POST", "/api/users/lists/favorite"), json={"listId": list_id}, auth=True
        )
        return res

    async def unfavorite(self, *, list_id: str | None = None) -> bool:
        """Unfavorite a user list

        Endpoint `/api/users/lists/unfavorite`

        Parameters
        ----------
        list_id : str, optional
            The id of the user list to unfavorite, by default None

        Returns
        -------
        bool
            True if the user list was unfavorited, False otherwise
        """
        res: bool = await self._session.request(
            Route("POST", "/api/users/lists/unfavorite"), json={"listId": list_id}, auth=True
        )
        return res

    async def update(
        self, name: str = MISSING, is_public: bool = MISSING, *, list_id: str
    ) -> UserList:
        """Update a user list

        Endpoint `/api/users/lists/update`

        Parameters
        ----------
        name : str, optional
            The new name of the user list, by default MISSING
        is_public : bool, optional
            Whether the user list should be public, by default MISSING
        list_id : str, optional
            The id of the user list to update, by default None

        Returns
        -------
        UserList
            The updated user list
        """
        data = remove_dict_missing({"listId": list_id, "name": name, "public": is_public})

        res: IUserList = await self._session.request(
            Route("POST", "/api/users/lists/update"),
            json=data,
            auth=True,
        )
        return UserList(raw_user_list=res, client=self._client)

    async def create_from_public(self, name: str, *, list_id: str):
        res: IUserList = await self._session.request(
            Route("POST", "/api/users/lists/create-from-public"),
            json={"listId": list_id, "name": name},
            auth=True,
        )
        return UserList(raw_user_list=res, client=self._client)

    async def get_memberships(
        self,
        for_public: bool = False,
        limit: int = 30,
        since_id: str | None = None,
        until_id: str | None = None,
        *,
        list_id: str,
    ) -> list[UserListMembership]:
        data = {
            "listId": list_id,
            "forPublic": for_public,
            "limit": limit,
            "sinceId": since_id,
            "untilId": until_id,
        }

        raw_user_list_memberships: list[IUserListMembership] = await self._session.request(
            Route("POST", "/api/users/lists/get-memberships"),
            json=data,
            auth=True,
        )

        return [
            UserListMembership(raw_user_list_membership, client=self._client)
            for raw_user_list_membership in raw_user_list_memberships
        ]

    async def get_all_memberships(
        self,
        for_public: bool = False,
        limit: int = 30,
        since_id: str | None = None,
        until_id: str | None = None,
        *,
        list_id: str,
    ):
        data = {
            "listId": list_id,
            "forPublic": for_public,
            "limit": limit,
            "sinceId": since_id,
            "untilId": until_id,
        }
        pagination = Pagination[IUserListMembership](
            http_client=self._session,
            route=Route("POST", "/api/users/lists/get-memberships"),
            json=data,
            auth=True,
        )

        while pagination.is_final is False:
            raw_user_list_memberships = await pagination.next()
            for raw_user_list_membership in raw_user_list_memberships:
                yield UserListMembership(raw_user_list_membership, client=self._client)

    # ここからはusers/lists系じゃないが、ここにあってほしい物
    async def get_time_line(
        self,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
        since_date: int | None = None,
        until_date: int | None = None,
        include_renote_my_notes: bool = True,
        include_local_renotes: bool = True,
        with_renotes: bool = True,
        with_files: bool = True,
        *,
        list_id: str,
    ) -> list[Note]:
        return await self._client.note.action.get_time_line(
            list_id=list_id,
            limit=limit,
            since_id=since_id,
            until_id=until_id,
            since_date=since_date,
            until_date=until_date,
            include_renote_my_notes=include_renote_my_notes,
            include_local_renotes=include_local_renotes,
            with_renotes=with_renotes,
            with_files=with_files,
        )

    async def get_all_time_line(
        self,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
        since_date: int | None = None,
        until_date: int | None = None,
        include_renote_my_notes: bool = True,
        include_local_renotes: bool = True,
        with_renotes: bool = True,
        with_files: bool = True,
        *,
        list_id: str,
    ):
        async for i in self._client.note.action.get_all_time_line(
            list_id=list_id,
            limit=limit,
            since_id=since_id,
            until_id=until_id,
            since_date=since_date,
            until_date=until_date,
            include_renote_my_notes=include_renote_my_notes,
            include_local_renotes=include_local_renotes,
            with_renotes=with_renotes,
            with_files=with_files,
        ):
            yield i


class ClientUserListActions(SharedUserListActions):
    def __init__(
        self,
        list_id: str,
        *,
        session: HTTPClient,
        client: ClientManager,
    ):
        super().__init__(session=session, client=client)
        self.__list_id: str = list_id

    @override
    async def delete(self, *, list_id: str | None = None) -> bool:
        """Delete a user list

        Endpoint `/api/users/lists/delete`

        Parameters
        ----------
        list_id : str, optional
            The id of the user list to delete, by default None

        Returns
        -------
        bool
            True if the user list was deleted, False otherwise
        """
        list_id = list_id or self.__list_id

        return await super().delete(list_id=list_id)

    @override
    async def show(self, for_public: bool = False, *, list_id: str | None = None) -> UserList:
        """Show a user list

        Endpoint `/api/users/lists/show`

        Parameters
        ----------
        for_public : bool, optional
            Whether to show the user list for the public, by default False
        list_id : str, optional
            The id of the user list to show, by default None

        Returns
        -------
        UserList
            The user list
        """
        list_id = list_id or self.__list_id

        return await super().show(for_public=for_public, list_id=list_id)

    @override
    async def favorite(self, *, list_id: str | None = None) -> bool:
        """Favorite a user list

        Endpoint `/api/users/lists/favorite`

        Parameters
        ----------
        list_id : str, optional
            The id of the user list to favorite, by default None

        Returns
        -------
        bool
            True if the user list was favorited, False otherwise
        """
        list_id = list_id or self.__list_id

        return await super().favorite(list_id=list_id)

    @override
    async def unfavorite(self, *, list_id: str | None = None) -> bool:
        """Unfavorite a user list

        Endpoint `/api/users/lists/unfavorite`

        Parameters
        ----------
        list_id : str, optional
            The id of the user list to unfavorite, by default None

        Returns
        -------
        bool
            True if the user list was unfavorited, False otherwise
        """
        list_id = list_id or self.__list_id

        return await super().unfavorite(list_id=list_id)

    @override
    async def update(
        self, name: str = MISSING, is_public: bool = MISSING, *, list_id: str | None = None
    ) -> UserList:
        """Update a user list

        Endpoint `/api/users/lists/update`

        Parameters
        ----------
        name : str, optional
            The new name of the user list, by default MISSING
        is_public : bool, optional
            Whether the user list should be public, by default MISSING
        list_id : str, optional
            The id of the user list to update, by default None

        Returns
        -------
        UserList
            The updated user list
        """
        list_id = list_id or self.__list_id

        return await super().update(name=name, is_public=is_public, list_id=list_id)

    @override
    async def create_from_public(self, name: str, *, list_id: str | None = None):
        list_id = list_id or self.__list_id

        return await super().create_from_public(name=name, list_id=list_id)

    @override
    async def get_memberships(
        self,
        for_public: bool = False,
        limit: int = 30,
        since_id: str | None = None,
        until_id: str | None = None,
        *,
        list_id: str | None = None,
    ) -> list[UserListMembership]:
        list_id = list_id or self.__list_id

        return await super().get_memberships(
            for_public=for_public,
            limit=limit,
            since_id=since_id,
            until_id=until_id,
            list_id=list_id,
        )

    @override
    async def get_all_memberships(
        self,
        for_public: bool = False,
        limit: int = 30,
        since_id: str | None = None,
        until_id: str | None = None,
        *,
        list_id: str | None = None,
    ):
        list_id = list_id or self.__list_id

        async for i in super().get_all_memberships(
            for_public=for_public,
            limit=limit,
            since_id=since_id,
            until_id=until_id,
            list_id=list_id,
        ):
            yield i

    # ここからはusers/lists系じゃないが、ここにあってほしい物
    @override
    async def get_time_line(
        self,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
        since_date: int | None = None,
        until_date: int | None = None,
        include_renote_my_notes: bool = True,
        include_local_renotes: bool = True,
        with_renotes: bool = True,
        with_files: bool = True,
        *,
        list_id: str | None = None,
    ) -> list[Note]:
        list_id = list_id or self.__list_id

        return await super().get_time_line(
            limit=limit,
            since_id=since_id,
            until_id=until_id,
            since_date=since_date,
            until_date=until_date,
            include_renote_my_notes=include_renote_my_notes,
            include_local_renotes=include_local_renotes,
            with_renotes=with_renotes,
            with_files=with_files,
            list_id=list_id,
        )

    @override
    async def get_all_time_line(
        self,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
        since_date: int | None = None,
        until_date: int | None = None,
        include_renote_my_notes: bool = True,
        include_local_renotes: bool = True,
        with_renotes: bool = True,
        with_files: bool = True,
        *,
        list_id: str | None = None,
    ):
        list_id = list_id or self.__list_id

        async for i in super().get_all_time_line(
            limit=limit,
            since_id=since_id,
            until_id=until_id,
            since_date=since_date,
            until_date=until_date,
            include_renote_my_notes=include_renote_my_notes,
            include_local_renotes=include_local_renotes,
            with_renotes=with_renotes,
            with_files=with_files,
            list_id=list_id,
        ):
            yield i


class UserListActions(SharedUserListActions):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        super().__init__(session=session, client=client)

    async def create(self, name: str) -> UserList:
        """Create a new user list

        Endpoint `/api/users/lists/create`

        Parameters
        ----------
        name : str
            The name of the new user list

        Returns
        -------
        UserList
            The created user list
        """
        raw_user_list: IUserList = await self._session.request(
            Route("POST", "/api/users/lists/create"), json={"name": name}, auth=True
        )
        return UserList(raw_user_list=raw_user_list, client=self._client)
